from django import template
from django.conf import settings
from django.core import urlresolvers
from django.template import Context, Template
from django.utils.translation import get_language, ugettext_lazy as _
from livesettings import config_choice_values

register = template.Library()

def js_make_select_readonly(select):
    #This is really just a mini-template
    #select must be a jquery object
    return """
    select = %(select)s;
    value = select.attr("value");
    if (value){
        text = "";
        for (c in select.children()){
          if (select.children()[c].value == value){
            text = select.children()[c].innerHTML;
            break;
          }
        }
        select.before("<strong>" + text + "</strong><input type=\\"hidden\\" name=\\"" + select.attr("name") + "\\" value=\\"" + value + "\\" \>");
        select.remove();
    }
    """ % {'select':select}

register.simple_tag(js_make_select_readonly)

def edit_subtypes(product):
    output = '<ul>'
    subtypes = product.get_subtypes()
    for key, subtype_label in config_choice_values('PRODUCT', 'PRODUCT_TYPES'):
        app, subtype = key.split("::")
        is_config = "ConfigurableProduct" in subtypes
        if subtype in subtypes:
            output += '<li><a href="/admin/%s/%s/%s/">' % (app, subtype.lower(), product.pk) + _('Edit %(subtype)s') % {'subtype': subtype_label} + '</a></li>'
            if is_config or subtype=="ProductVariation":
                 output += '<li><a href="%s">Variation Manager</a></li>' % (urlresolvers.reverse("satchmo_admin_variation_manager", args = [product.id]))
        else:
            if not(is_config and subtype=="ProductVariation"):
                output += ' <li><a href="/admin/%s/%s/add/?product=%s">' %(app, subtype.lower(), product.pk) + _('Add %(subtype)s') % {'subtype': subtype_label} + '</a></li>'

    output += '</ul>'
    return output

register.simple_tag(edit_subtypes)

def list_variations(configurableproduct):
    opts = configurableproduct.get_all_options()
    output = "{% load admin_modify adminmedia %}"
    output += "<table>"
    for p_opt in opts:
        opt_strs = []
        [opt_strs.append(opt.name) for opt in p_opt]
        opt_str = ', '.join(opt_strs)

        product = configurableproduct.get_product_from_options(p_opt)
        if product:
            p_url = '/admin/product/product/%s/' % product.pk
            pv_url = '/admin/product/productvariation/%s/delete/' % product.pk
            output += """
            <tr>
            <td>%s</td>
            <td><a href="%s">%s</a></td>
            <td><a class="deletelink" href="%s">%s</a></td>
            </tr>
            """ % (opt_str, p_url, product.slug, pv_url,
                _("Delete ProductVariation"))
        else:
            #opt_pks = [str(opt.pk) for opt in p_opt]
            #opt_pks = ','.join(opt_pks)
            # TODO [NFA]: Blocked by Django ticket #7738.
            opt_pks = ''
            add_url = ('/admin/product/productvariation/add/' +
                "?product=%s&parent=%s&options=%s" % (
                configurableproduct.product.pk, configurableproduct.product.pk,
                opt_pks))
            output += """
            <tr>
            <td>%s</td>
            <td/>
            <td><a href="%s" class="add-another" id="add_productvariation"> <img src="{%% admin_media_prefix %%}img/admin/icon_addlink.gif" width="10" height="10" alt="Add ProductVariation"/> Add Variation</a></td>
            </tr>
            """ % (opt_str, add_url)
    output += "</table>"
    t = Template(output)
    return t.render(Context())

register.simple_tag(list_variations)




from django.core.management.base import NoArgsCommand

class Command(NoArgsCommand):
    help = "Load sample store data for satchmo."

    def handle_noargs(self, **options):
        from satchmo_store.contact.models import (
            AddressBook,
            Contact,
            ContactOrganization,
            ContactOrganizationRole,
            ContactRole,
            Organization,
            PhoneNumber,
        )
        from product.models import (
            Category,
            OptionGroup,
            Option,
            Price,
            Product,
        )
        from product.modules.configurable.models import ConfigurableProduct

        from satchmo_store.shop.models import Config
        from django.conf import settings
        from l10n.models import Country
        from django.contrib.sites.models import Site
        from django.contrib.auth.models import User
        #idempotency test

        print "Checking for existing sample data."
        try:
            p = Product.objects.get(slug="dj-rocks")
            print "It looks like you already have loaded the sample store data, quitting."
            import sys
            sys.exit(1)
        except Product.DoesNotExist:
            pass

        print "Loading sample store data."

        #Load basic configuration information

        print "Creating site..."
        try:
            site = Site.objects.get(id=settings.SITE_ID)
            print "Using existing site #%i" % settings.SITE_ID
        except Site.DoesNotExist:
            print "Creating Example Store Site"
            site = Site(domain="localhost", name="Sample Store")
        site.domain = settings.SITE_DOMAIN
        site.name = settings.SITE_NAME
        site.save()
        store_country = Country.objects.get(iso3_code='USA')
        config = Config(site=site, store_name=settings.SITE_NAME, country=store_country, sales_country=store_country)
        config.save()
        config.shipping_countries.add(store_country)
        config.save()
        print "Creating Customers..."
        # Import some customers

        customer = ContactRole.objects.get(pk='Customer')
        c1 = Contact(first_name="Chris", last_name="Smith", email="chris@aol.com", role=customer, notes="Really cool stuff")
        c1.save()
        p1 = PhoneNumber(contact=c1, phone="601-555-5511", type="Home",primary=True)
        p1.save()
        c2 = Contact(first_name="John", last_name="Smith", email="abc@comcast.com", role=customer, notes="Second user")
        c2.save()
        p2 = PhoneNumber(contact=c2, phone="999-555-5111", type="Work",primary=True)
        p2.save()
        # Import some addresses for these customers
        us = Country.objects.get(iso2_code='US');
        a1 = AddressBook(description="Home", street1="8235 Pike Street", city="Anywhere Town", state="TN",
                     postal_code="38138", country=us, is_default_shipping=True, contact=c1)
        a1.save()
        a2 = AddressBook(description="Work", street1="1245 Main Street", city="Stillwater", state="MN",
                     postal_code="55082", country=us, is_default_shipping=True, contact=c2)
        a2.save()
        print "Creating Suppliers..."
        #Import some suppliers

        supplier = ContactOrganizationRole.objects.get(pk='Supplier')
        company = ContactOrganization.objects.get(pk='Company')
        contactsupplier = ContactRole.objects.get(pk='Supplier')
        org1 = Organization(name="Rhinestone Ronny", type=company, role=supplier)
        org1.save()
        c4 = Contact(first_name="Fred", last_name="Jones", email="fj@rr.com",
            role=contactsupplier, organization=org1)
        c4.save()
        p4 = PhoneNumber(contact=c4,phone="800-188-7611", type="Work", primary=True)
        p4.save()
        p5 = PhoneNumber(contact=c4,phone="755-555-1111",type="Fax")
        p5.save()
        a3 = AddressBook(contact=c4, description="Mailing address", street1="Receiving Dept",
            street2="918 Funky Town St", city="Fishkill",
             state="NJ", country=us, postal_code="19010")
        a3.save()
        #s1 = Supplier(name="Rhinestone Ronny", address1="918 Funky Town St", address2="Suite 200",
        #              city="Fishkill", state="NJ", zip="19010", phone1="800-188-7611", fax="900-110-1909", email="ron@rhinestone.com",
        #              notes="My main supplier")
        #s1.save()

        #s2 = Supplier(name="Shirt Sally", address1="9 ABC Lane",
        #    city="Happyville", state="MD", zip="190111", phone1="888-888-1111", fax="999-110-1909", email="sally@shirts.com",
        #              notes="Shirt Supplier")
        #s2.save()


        print "Creating Categories..."
        #Create some categories
        cat1 = Category(site=site, name="Shirts",slug="shirts",description="Women's Shirts")
        cat1.save()
        cat2 = Category(site=site, name="Short Sleeve",slug="shortsleeve",description="Short sleeve shirts", parent=cat1)
        cat2.save()
        cat3 = Category(site=site, name="Books",slug="book",description="Books")
        cat3.save()
        cat4 = Category(site=site, name="Fiction",slug="fiction",description="Fiction Books", parent=cat3)
        cat4.save()
        cat5 = Category(site=site, name="Science Fiction",slug="scifi",description="Science Fiction",parent=cat4)
        cat5.save()
        cat6 = Category(site=site, name="Non Fiction",slug="nonfiction",description="Non Fiction",parent=cat3)
        cat6.save()
        cat7 = Category(site=site, name="Software", slug="software")
        cat7.save()


        print "Creating products..."
        #Create some items
        i1 = Product(site=site, name="Django Rocks shirt", slug="dj-rocks", description="Really cool shirt",
                 active=True, featured=True)
        i1.save()
        p1 = Price(price="20.00", product=i1)
        p1.save()
        i1.category.add(cat1)
        i1.save()
        i2 = Product(site=site, name="Python Rocks shirt", slug="PY-Rocks",
            description="Really cool python shirt - One Size Fits All",
            active=True, featured=True)
        i2.save()
        p2 = Price(price="19.50", product=i2)
        p2.save()
        i2.category.add(cat2)
        i2.save()
        i3 = Product(site=site, name="A really neat book", slug="neat-book",
            description="A neat book.  You should buy it.",
            active=True, featured=True)
        i3.save()
        p3 = Price(price="5.00", product=i3)
        p3.save()
        i3.category.add(cat4)
        i3.save()
        i4 = Product(site=site, name="Robots Attack!", slug="robot-attack",
            description="Robots try to take over the world.",
            active=True, featured=True)
        i4.save()
        p4 = Price(price="7.99", product=i4)
        p4.save()
        i4.category.add(cat5)
        i4.save()

    #    i5 = Product(site=site, name="Really Neat Software", slug="neat-software", description="Example Configurable/Downloadable product", active=True, featured=True)
    #    i5.save()
    #    i5.category.add(cat7)
    #    i5.save()

        #Create an attribute set
        optSet1 = OptionGroup(site=site, name="sizes", sort_order=1)
        optSet2 = OptionGroup(site=site, name="colors", sort_order=2)
        optSet1.save()
        optSet2.save()

        optSet3 = OptionGroup(site=site, name="Book type", sort_order=1)
        optSet3.save()

        optSet4 = OptionGroup(site=site, name="Full/Upgrade", sort_order=5)
        optSet4.save()

        optItem1a = Option(name="Small", value="S", sort_order=1, option_group=optSet1)
        optItem1a.save()
        optItem1b = Option(name="Medium", value="M", sort_order=2, option_group=optSet1)
        optItem1b.save()
        optItem1c = Option(name="Large", value="L", sort_order=3, price_change="1.00", option_group=optSet1)
        optItem1c.save()

        optItem2a = Option(name="Black", value="B", sort_order=1, option_group=optSet2)
        optItem2a.save()
        optItem2b = Option(name="White", value="W", sort_order=2, option_group=optSet2)
        optItem2b.save()
        optItem2c = Option(name="Blue", value="BL", sort_order=3, price_change="2.00", option_group=optSet2)
        optItem2c.save()

        optItem3a = Option(name="Hard cover", value="hard", sort_order=1, option_group=optSet3)
        optItem3a.save()
        optItem3b = Option(name="Soft cover", value="soft", sort_order=2, price_change="1.00", option_group=optSet3)
        optItem3b.save()
        optItem3c = Option(name="On tape", value="tape", sort_order=3, option_group=optSet3)
        optItem3c.save()

        optItem4a = Option(name="Full Version", value="full", option_group=optSet4, sort_order=1)
        optItem4a.save()
        optItem4b = Option(name="Upgrade Version", value="upgrade", option_group=optSet4, sort_order=2)
        optItem4b.save()


        #Add the option group to our items
        pg1 = ConfigurableProduct(product=i1)
        pg1.save()
        pg1.option_group.add(optSet1)
        pg1.save()
        pg1.option_group.add(optSet2)
        pg1.save()

        pg3 = ConfigurableProduct(product=i3)
        pg3.save()
        pg3.option_group.add(optSet3)
        pg3.save()

        pg4 = ConfigurableProduct(product=i4)
        pg4.save()
        pg4.option_group.add(optSet3)
        pg4.save()

    #    pg5 = ConfigurableProduct(product=i5)
    #    pg5.option_group.add(optSet4)
    #    pg5.save()

        print "Creating product variations..."
        #Create the required sub_items
        pg1.create_all_variations()
        pg3.create_all_variations()
        pg4.create_all_variations()
        #pg5.create_all_variations()

        #set prices for full and upgrade versions of neat-software, this is an alternative to using the price_change in options, it allows for more flexability when required.
    #    pv1 = pg5.get_product_from_options([optItem4a])
    #    Price(product=pv1, price='5.00').save()
    #    Price(product=pv1, price='2.00', quantity='50.00').save()
    #    DownloadableProduct(product=pv1).save()

    #    pv2 = pg5.get_product_from_options([optItem4b])
    #    Price(product=pv2, price='1.00').save()
    #    DownloadableProduct(product=pv2).save()

        print "Create a test user..."
        #First see if our test user is still there, then use or create that user
        try:
            test_user = User.objects.get(username="csmith")
        except:
            test_user = User.objects.create_user('csmith', 'tester@testsite.com', 'test')
            test_user.save()
        c1.user = test_user
        c1.save()



"""
Signals for contacts.

Signals:

 - `satchmo_contact_view`: called like so::

    signals.satchmo_contact_view.send(sender, contact=user_data, contact_dict=contact_dict)

 - `satchmo_contact_location_changed`
"""

import django.dispatch

satchmo_contact_location_changed = django.dispatch.Signal()
satchmo_contact_view = django.dispatch.Signal()
validate_postcode = django.dispatch.Signal()




'''
Elixir package

A declarative layer on top of the `SQLAlchemy library
<http://www.sqlalchemy.org/>`_. It is a fairly thin wrapper, which provides
the ability to create simple Python classes that map directly to relational
database tables (this pattern is often referred to as the Active Record design
pattern), providing many of the benefits of traditional databases
without losing the convenience of Python objects.

Elixir is intended to replace the ActiveMapper SQLAlchemy extension, and the
TurboEntity project but does not intend to replace SQLAlchemy's core features,
and instead focuses on providing a simpler syntax for defining model objects
when you do not need the full expressiveness of SQLAlchemy's manual mapper
definitions.
'''

try:
    set
except NameError:
    from sets import Set as set

import sqlalchemy
from sqlalchemy.types import *

from elixir.options import using_options, using_table_options, \
                           using_mapper_options, options_defaults, \
                           using_options_defaults
from elixir.entity import Entity, EntityBase, EntityMeta, EntityDescriptor, \
                          setup_entities, cleanup_entities
from elixir.fields import has_field, Field
from elixir.relationships import belongs_to, has_one, has_many, \
                                 has_and_belongs_to_many, \
                                 ManyToOne, OneToOne, OneToMany, ManyToMany
from elixir.properties import has_property, GenericProperty, ColumnProperty, \
                              Synonym
from elixir.statements import Statement
from elixir.collection import EntityCollection, GlobalEntityCollection


__version__ = '0.7.1'

__all__ = ['Entity', 'EntityBase', 'EntityMeta', 'EntityCollection',
           'entities',
           'Field', 'has_field',
           'has_property', 'GenericProperty', 'ColumnProperty', 'Synonym',
           'belongs_to', 'has_one', 'has_many', 'has_and_belongs_to_many',
           'ManyToOne', 'OneToOne', 'OneToMany', 'ManyToMany',
           'using_options', 'using_table_options', 'using_mapper_options',
           'options_defaults', 'using_options_defaults',
           'metadata', 'session',
           'create_all', 'drop_all',
           'setup_all', 'cleanup_all',
           'setup_entities', 'cleanup_entities'] + \
           sqlalchemy.types.__all__

__doc_all__ = ['create_all', 'drop_all',
               'setup_all', 'cleanup_all',
               'metadata', 'session']

# default session
session = sqlalchemy.orm.scoped_session(sqlalchemy.orm.sessionmaker())

# default metadata
metadata = sqlalchemy.MetaData()

metadatas = set()

# default entity collection
entities = GlobalEntityCollection()


def create_all(*args, **kwargs):
    '''Create the necessary tables for all declared entities'''
    for md in metadatas:
        md.create_all(*args, **kwargs)


def drop_all(*args, **kwargs):
    '''Drop tables for all declared entities'''
    for md in metadatas:
        md.drop_all(*args, **kwargs)


def setup_all(create_tables=False, *args, **kwargs):
    '''Setup the table and mapper of all entities in the default entity
    collection.

    This is called automatically if any entity of the collection is configured
    with the `autosetup` option and it is first accessed,
    instanciated (called) or the create_all method of a metadata containing
    tables from any of those entities is called.
    '''
    setup_entities(entities)

    # issue the "CREATE" SQL statements
    if create_tables:
        create_all(*args, **kwargs)


def cleanup_all(drop_tables=False, *args, **kwargs):
    '''Clear all mappers, clear the session, and clear all metadatas.
    Optionally drops the tables.
    '''
    session.close()

    cleanup_entities(entities)

    sqlalchemy.orm.clear_mappers()
    entities.clear()

    if drop_tables:
        drop_all(*args, **kwargs)

    for md in metadatas:
        md.clear()
    metadatas.clear()





from satchmo_store.contact.models import Organization, Contact, Interaction, PhoneNumber, AddressBook, ContactOrganization, ContactOrganizationRole, ContactRole, ContactInteractionType
from satchmo_utils.admin import AutocompleteAdmin
from django.contrib import admin
from django.utils.translation import get_language, ugettext_lazy as _

class Contact_Inline(admin.TabularInline):
    model = Contact
    extra = 1

class PhoneNumber_Inline(admin.TabularInline):
    model = PhoneNumber
    extra = 1

class AddressBook_Inline(admin.StackedInline):
    model = AddressBook
    extra = 1


class OrganizationOptions(admin.ModelAdmin):
    list_filter = ['type', 'role']
    list_display = ['name', 'type', 'role']

class ContactOptions(AutocompleteAdmin):
    list_display = ('last_name', 'first_name', 'organization', 'role')
    list_filter = ['create_date', 'role', 'organization']
    ordering = ['last_name']
    search_fields = ('first_name', 'last_name', 'email') 
    related_search_fields = {'user': ('username', 'first_name', 'last_name', 'email')} 
    related_string_functions = {'user': lambda u: u"%s (%s)" % (u.username, u.get_full_name())}
    inlines = [PhoneNumber_Inline, AddressBook_Inline]

class InteractionOptions(admin.ModelAdmin):
    list_filter = ['type', 'date_time']

class ContactOrganizationOptions(admin.ModelAdmin):
    pass
    
class ContactRoleOptions(admin.ModelAdmin):
    pass
    
class ContactOrganizationRoleOptions(admin.ModelAdmin):
    pass

class ContactInteractionTypeOptions(admin.ModelAdmin):
    pass

admin.site.register(Organization, OrganizationOptions)
admin.site.register(Contact, ContactOptions)
admin.site.register(Interaction, InteractionOptions)
admin.site.register(ContactOrganization, ContactOrganizationOptions)
admin.site.register(ContactOrganizationRole, ContactOrganizationRoleOptions)
admin.site.register(ContactRole, ContactRoleOptions)
admin.site.register(ContactInteractionType, ContactInteractionTypeOptions)



from django import http
from django.shortcuts import render_to_response
from django.template import Context, RequestContext
from django.template.loader import get_template
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from livesettings import config_get_group, config_value
from payment.config import gateway_live
from payment.views import confirm, payship
from satchmo_store.shop.models import Order
from satchmo_utils.dynamic import lookup_url
from satchmo_utils.views import bad_or_missing
import auth
import base64
import hmac
import logging
import notifications
import sha

# TODO: This module doesn't seem to actually record any payments.

log = logging.getLogger("payment.modules.google.processor")

class GoogleCart(object):
    def __init__(self, order, payment_module, live):
        self.settings = payment_module
        self.cart_xml = self._cart_xml(order)
        self.signature = self._signature(live)

    def _cart_xml(self, order):
        template = get_template(self.settings["CART_XML_TEMPLATE"].value)

        shopping_url = lookup_url(self.settings, 'satchmo_checkout-success', True, self.settings.SSL.value)
        edit_url = lookup_url(self.settings, 'satchmo_cart', True, self.settings.SSL.value)
        ctx = Context({"order" : order,
                       "continue_shopping_url" : shopping_url,
                       "edit_cart_url" : edit_url,
                       "currency" : self.settings.CURRENCY_CODE.value,
                       })
        return template.render(ctx)

    def _signature(self, live):
        if live:
            merchkey = self.settings.MERCHANT_KEY.value
        else:
            merchkey = self.settings.MERCHANT_TEST_KEY.value

        s = hmac.new(str(merchkey), self.cart_xml, sha)
        rawsig = s.digest()
        return rawsig

    def encoded_cart(self):
        return base64.encodestring(self.cart_xml)[:-1]

    def encoded_signature(self):
        sig = base64.encodestring(self.signature)[:-1]
        log.debug("Sig is: %s", sig)
        return sig

@never_cache
def pay_ship_info(request):
    return payship.simple_pay_ship_info(request, config_get_group('PAYMENT_GOOGLE'), 'shop/checkout/google/pay_ship.html')

@never_cache
def confirm_info(request):
    payment_module = config_get_group('PAYMENT_GOOGLE')

    controller = confirm.ConfirmController(request, payment_module)
    if not controller.sanity_check():
        return controller.response

    live = gateway_live(payment_module)
    gcart = GoogleCart(controller.order, payment_module, live)
    log.debug("CART:\n%s", gcart.cart_xml)
        
    post_url = auth.get_url()
    default_view_tax = config_value('TAX', 'DEFAULT_VIEW_TAX')
    
    ctx = {
        'post_url': post_url,
        'google_cart' : gcart.encoded_cart(),
        'google_signature' : gcart.encoded_signature(),
        'PAYMENT_LIVE' : live
    }

    controller.extra_context = ctx
    controller.confirm()
    return controller.response

@never_cache
def notification(request):
    """
    View to receive notifications from google about order status
    """
    data = request.POST
    log.debug(data)
    
    # check the given merchant id
    log.debug("Google Checkout Notification")
    response = auth.do_auth(request)
    if response:
        return response
    
    # ok its authed, get the type and serial
    type = data['_type']
    serial_number = data['serial-number'].strip()
    
    log.debug("type: %s" % type)
    log.debug("serial-number: %s" % serial_number)
    
    # check type
    if type == 'new-order-notification':
        notifications.notify_neworder(request, data)
    elif type == 'order-state-change-notification':
        notifications.notify_statechanged(request, data)
    elif type == 'charge-amount-notification':
        notifications.notify_chargeamount(request, data)

    # return ack so google knows we handled the message
    ack = '<notification-acknowledgment xmlns="http://checkout.google.com/schema/2" serial-number="%s"/>' % serial_number 
    response = http.HttpResponse(content=ack, content_type="text/xml; charset=UTF-8")
    log.debug(response)
    return response

@never_cache
def success(request):
    """
    The order has been succesfully processed.  This can be used to generate a receipt or some other confirmation
    """
    try:
        order = Order.objects.from_request(request)
    except Order.DoesNotExist:
        return bad_or_missing(request, _('Your order has already been processed.'))
        
    del request.session['orderID']
    context = RequestContext(request, {'order': order})
    return render_to_response('shop/checkout/success.html',
                              context_instance=context)




"""
URLConf for Satchmo Newsletter app

This will get automatically added by satchmo_store, under the url given in your livesettings "NEWSLETTER","NEWSLETTER_SLUG"
"""

from django.conf.urls.defaults import *
from livesettings import config_value
import logging
log = logging.getLogger('newsletter.urls')

urlpatterns = patterns('satchmo_ext.newsletter.views',
    (r'^subscribe/$', 'add_subscription', {}, 'newsletter_subscribe'),
    (r'^subscribe/ajah/$', 'add_subscription', 
        {'result_template' : 'newsletter/ajah.html'}, 'newsletter_subscribe_ajah'),
    (r'^unsubscribe/$', 'remove_subscription', 
        {}, 'newsletter_unsubscribe'),
    (r'^unsubscribe/ajah/$', 'remove_subscription', 
        {'result_template' : 'newsletter/ajah.html'}, 'newsletter_unsubscribe_ajah'),
    (r'^update/$', 'update_subscription', {}, 'newsletter_update'),
)

newsbase = r'^' + config_value('NEWSLETTER','NEWSLETTER_SLUG') + '/'    
newspatterns = patterns('',
    (newsbase, include('satchmo_ext.newsletter.urls'))
)

def add_newsletter_urls(sender, patterns=(), **kwargs):
    log.debug("Adding newsletter urls at %s", newsbase)
    patterns += newspatterns



from django import template
from satchmo_store.shop import get_satchmo_setting
from satchmo_store.shop.models import Order, ORDER_STATUS
from satchmo_store.shop.utils import is_multihost_enabled

register = template.Library()

def customorder_management(order):
    custom = []
    for orderitem in order.orderitem_set.all():
        if 'CustomProduct' in orderitem.product.get_subtypes():
            custom.append(orderitem)

    return {
        'SHOP_BASE' : get_satchmo_setting('SHOP_BASE'),
        'customitems' : custom
    }

register.inclusion_tag('shop/admin/_customorder_management.html')(customorder_management)

def inprocess_order_list():
    """Returns a formatted list of in-process orders"""
    inprocess = unicode(ORDER_STATUS[2][0])
    orders = orders_at_status(inprocess)

    return {
        'orders' : orders,
        'multihost' : is_multihost_enabled()
    }

register.inclusion_tag('shop/admin/_ordercount_list.html')(inprocess_order_list)

def orders_at_status(status):
    return Order.objects.filter(status=status).order_by('-time_stamp')

def orderpayment_list(order):
    return {
        'SHOP_BASE' : get_satchmo_setting('SHOP_BASE'),
        'order' : order,
        'payments' : order.payments.all()
        }

register.inclusion_tag('shop/admin/_orderpayment_list.html')(orderpayment_list)

def pending_order_list():
    """Returns a formatted list of pending orders"""
    pending = unicode(ORDER_STATUS[1][0])
    orders = orders_at_status(pending)

    return {
        'orders' : orders,
        'multihost' : is_multihost_enabled()
    }

register.inclusion_tag('shop/admin/_ordercount_list.html')(pending_order_list)



from livesettings import *
from django.utils.translation import ugettext_lazy as _
from product.config import PRODUCT_GROUP
config_register(
IntegerValue(PRODUCT_GROUP,
    'RECENT_MAX',
    description=_("Maximum recent items"),
    help_text=_("""The maximum number of items show in the recent box."""),
    default=4,
    ),
)



"""Causes the keyedcache to also use a first-level cache in memory - this can cut 30-40% of memcached calls.

To enable, add this to some models.py file in an app::

    from keyedcache import threaded
    threaded.start_listening()

"""
from threaded_multihost import threadlocals
from django.core.signals import request_started, request_finished
from keyedcache import cache_clear_request, cache_use_request_caching, cache_set_request_uid
import random
import logging
log = logging.getLogger('keyedcache.threaded')

def set_request_uid(sender, *args, **kwargs):
    """Puts a unique id into the thread"""
    tid = random.randrange(1,10000000)
    threadlocals.set_thread_variable('request_uid', tid)
    #log.debug('request UID: %s', tid)
    
def clear_request_uid(sender, *args, **kwargs):
    """Removes the thread cache for this request"""
    tid = threadlocals.get_thread_variable('request_uid', -1)
    if tid > -1:
        cache_clear_request(tid)
    
def start_listening():
    log.debug('setting up threaded keyedcache')
    cache_use_request_caching()
    request_started.connect(set_request_uid)
    request_finished.connect(clear_request_uid)



from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from satchmo_store.shop.models import Order
from satchmo_utils.views import bad_or_missing

def success(request):
    """
    The order has been succesfully processed.  This can be used to generate a receipt or some other confirmation
    """
    try:
        order = Order.objects.from_request(request)
    except Order.DoesNotExist:
        return bad_or_missing(request, _('Your order has already been processed.'))

    del request.session['orderID']
    return render_to_response('shop/checkout/success.html',
                              {'order': order},
                              context_instance=RequestContext(request))
success = never_cache(success)

def failure(request):
    return render_to_response(
        'shop/checkout/failure.html',
        {},
        context_instance=RequestContext(request)
    )



# Copyright (C) 2004,2005 by SICEm S.L.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

def enum_to_string(enum_value, pspec=None, enum=None):
    if pspec:
        enum_class = pspec.enum_class
    elif enum:
        enum_class = enum
    else:
        raise AssertionError

    if enum_class is not None:
        for value, enum in list(enum_class.__enum_values__.items()):
            if value == enum_value:
                return enum.value_nick
    return ""

def flags_to_string(flags_value, pspec=None, flags=None):
    """Convert a flag value (integer) to a string, using the pspec
    for introspection.
    It tries fairly hard to return a string as short as possible"""

    def isflag(n):
        """Helper to find the number of 1 in an integer.
        We're only interested in numbers with exactly one 1.
        """

        ones = 0
        i = 0
        while 1:
            # The value is too large, skip out
            if (1 << i) > n:
                break

            # Check if the ith bit of n is set
            if (n >> i) & 1 == 1:
                ones += 1

            if ones > 1:
                return False

            i += 1

        return ones == 1

    if flags_value == 0:
        return ""

    if pspec:
        flags_class = pspec.flags_class
    elif flags:
        flags_class = flags
    else:
        raise AssertionError

    flags_names = []
    for value, flag in list(flags_class.__flags_values__.items()):
        if not isflag(value):
            continue

        if (value & flags_value) == 0:
            continue

        for nick in flag.value_nicks:
            if nick in flags_names:
                continue

            flags_names.append(nick)

    flags_names.sort()
    retval = ' | '.join(flags_names)
    return retval



'''
This module provides support for defining the fields (columns) of your
entities. Elixir currently supports two syntaxes to do so: the default
`Attribute-based syntax`_ as well as the has_field_ DSL statement.

Attribute-based syntax
----------------------

Here is a quick example of how to use the object-oriented syntax.

.. sourcecode:: python

    class Person(Entity):
        id = Field(Integer, primary_key=True)
        name = Field(String(50), required=True)
        ssn = Field(String(50), unique=True)
        biography = Field(Text)
        join_date = Field(DateTime, default=datetime.datetime.now)
        photo = Field(Binary, deferred=True)
        _email = Field(String(20), colname='email', synonym='email')

        def _set_email(self, email):
           self._email = email
        def _get_email(self):
           return self._email
        email = property(_get_email, _set_email)


The Field class takes one mandatory argument, which is its type. Please refer
to SQLAlchemy documentation for a list of `types supported by SQLAlchemy
<http://www.sqlalchemy.org/docs/05/reference/sqlalchemy/types.html>`_.

Following that first mandatory argument, fields can take any number of
optional keyword arguments. Please note that all the **arguments** that are
**not specifically processed by Elixir**, as mentioned in the documentation
below **are passed on to the SQLAlchemy ``Column`` object**. Please refer to
the `SQLAlchemy Column object's documentation
<http://www.sqlalchemy.org/docs/05/reference/sqlalchemy/schema.html
#sqlalchemy.schema.Column>`_ for more details about other
supported keyword arguments.

The following Elixir-specific arguments are supported:

+-------------------+---------------------------------------------------------+
| Argument Name     | Description                                             |
+===================+=========================================================+
| ``required``      | Specify whether or not this field can be set to None    |
|                   | (left without a value). Defaults to ``False``, unless   |
|                   | the field is a primary key.                             |
+-------------------+---------------------------------------------------------+
| ``colname``       | Specify a custom name for the column of this field. By  |
|                   | default the column will have the same name as the       |
|                   | attribute.                                              |
+-------------------+---------------------------------------------------------+
| ``deferred``      | Specify whether this particular column should be        |
|                   | fetched by default (along with the other columns) when  |
|                   | an instance of the entity is fetched from the database  |
|                   | or rather only later on when this particular column is  |
|                   | first referenced. This can be useful when one wants to  |
|                   | avoid loading a large text or binary field into memory  |
|                   | when its not needed. Individual columns can be lazy     |
|                   | loaded by themselves (by using ``deferred=True``)       |
|                   | or placed into groups that lazy-load together (by using |
|                   | ``deferred`` = `"group_name"`).                         |
+-------------------+---------------------------------------------------------+
| ``synonym``       | Specify a synonym name for this field. The field will   |
|                   | also be usable under that name in keyword-based Query   |
|                   | functions such as filter_by. The Synonym class (see the |
|                   | `properties` module) provides a similar functionality   |
|                   | with an (arguably) nicer syntax, but a limited scope.   |
+-------------------+---------------------------------------------------------+

has_field
---------

The `has_field` statement allows you to define fields one at a time.

The first argument is the name of the field, the second is its type. Following
these, any number of keyword arguments can be specified for additional
behavior. The following arguments are supported:

+-------------------+---------------------------------------------------------+
| Argument Name     | Description                                             |
+===================+=========================================================+
| ``through``       | Specify a relation name to go through. This field will  |
|                   | not exist as a column on the database but will be a     |
|                   | property which automatically proxy values to the        |
|                   | ``attribute`` attribute of the object pointed to by the |
|                   | relation. If the ``attribute`` argument is not present, |
|                   | the name of the current field will be used. In an       |
|                   | has_field statement, you can only proxy through a       |
|                   | belongs_to or an has_one relationship.                  |
+-------------------+---------------------------------------------------------+
| ``attribute``     | Name of the "endpoint" attribute to proxy to. This      |
|                   | should only be used in combination with the ``through`` |
|                   | argument.                                               |
+-------------------+---------------------------------------------------------+


Here is a quick example of how to use ``has_field``.

.. sourcecode:: python

    class Person(Entity):
        has_field('id', Integer, primary_key=True)
        has_field('name', String(50))
'''
from sqlalchemy import Column
from sqlalchemy.orm import deferred, synonym
from sqlalchemy.ext.associationproxy import association_proxy

from elixir.statements import ClassMutator
from elixir.properties import Property

__doc_all__ = ['Field']


class Field(Property):
    '''
    Represents the definition of a 'field' on an entity.

    This class represents a column on the table where the entity is stored.
    '''

    def __init__(self, type, *args, **kwargs):
        super(Field, self).__init__()

        self.colname = kwargs.pop('colname', None)
        self.synonym = kwargs.pop('synonym', None)
        self.deferred = kwargs.pop('deferred', False)
        if 'required' in kwargs:
            kwargs['nullable'] = not kwargs.pop('required')
        self.type = type
        self.primary_key = kwargs.get('primary_key', False)

        self.column = None
        self.property = None

        self.args = args
        self.kwargs = kwargs

    def attach(self, entity, name):
        # If no colname was defined (through the 'colname' kwarg), set
        # it to the name of the attr.
        if self.colname is None:
            self.colname = name
        super(Field, self).attach(entity, name)

    def create_pk_cols(self):
        if self.primary_key:
            self.create_col()

    def create_non_pk_cols(self):
        if not self.primary_key:
            self.create_col()

    def create_col(self):
        self.column = Column(self.colname, self.type,
                             *self.args, **self.kwargs)
        self.add_table_column(self.column)

    def create_properties(self):
        if self.deferred:
            group = None
            if isinstance(self.deferred, basestring):
                group = self.deferred
            self.property = deferred(self.column, group=group)
        elif self.name != self.colname:
            # if the property name is different from the column name, we need
            # to add an explicit property (otherwise nothing is needed as it's
            # done automatically by SA)
            self.property = self.column

        if self.property is not None:
            self.add_mapper_property(self.name, self.property)

        if self.synonym:
            self.add_mapper_property(self.synonym, synonym(self.name))


def has_field_handler(entity, name, *args, **kwargs):
    if 'through' in kwargs:
        setattr(entity, name,
                association_proxy(kwargs.pop('through'),
                                  kwargs.pop('attribute', name),
                                  **kwargs))
        return
    field = Field(*args, **kwargs)
    field.attach(entity, name)

has_field = ClassMutator(has_field_handler)



from django.conf import settings
from django.core import urlresolvers
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from product.modules.downloadable.models import DownloadLink
from satchmo_store.shop.signals import sendfile_url_for_file
import mimetypes

import os
import os.path
import re
from urlparse import urljoin

SHA1_RE = re.compile('^[a-f0-9]{40}$')

def _validate_key(download_key):
    """
    Helper function to make sure the key is valid and all the other constraints on
    the download are still valid.
    Returns a tuple (False,"Error Message", None) or (True, None, dl_product)
    """
    download_key = download_key.lower()
    if not SHA1_RE.search(download_key):
        error_message = _("The download key is invalid.")
        return (False, error_message, None)
    try:
        dl_product = DownloadLink.objects.get(key=download_key)
    except:
        error_message = _("The download key is invalid.")
        return (False, error_message, None)
    valid, msg = dl_product.is_valid()
    if not valid:
        return (False, msg, None)
    else:
        return (True, None, dl_product)

def process(request, download_key):
    """
    Validate that the key is good, then set a session variable.
    Redirect to the download view.

    We use this two step process so that we can easily display meaningful feedback
    to the user.
    """
    valid, msg, dl_product = _validate_key(download_key)
    if not valid:
        context = RequestContext(request, {'error_message': msg})
        return render_to_response('shop/download.html',
                                  context_instance=context)
    else:
        # The key is valid so let's set the session variable and redirect to the
        # download view
        request.session['download_key'] = download_key
        url = urlresolvers.reverse('satchmo_download_send', kwargs= {'download_key': download_key})
        context = RequestContext(request, {'download_product': dl_product,
                                            'dl_url' : url})
        return render_to_response('shop/download.html', context_instance=context)

def send_file(request, download_key):
    """
    After the appropriate session variable has been set, we commence the download.
    The key is maintained in the url but the session variable is used to control the
    download in order to maintain security.
    """
    if not request.session.get('download_key', False):
        url = urlresolvers.reverse('satchmo_download_process', kwargs = {'download_key': download_key})
        return HttpResponseRedirect(url)
    valid, msg, dl_product = _validate_key(request.session['download_key'])
    if not valid:
        url = urlresolvers.reverse('satchmo_download_process', kwargs = {'download_key': request.session['download_key']})
        return HttpResponseRedirect(url)

    # some temp vars
    file = dl_product.downloadable_product.file
    file_url = '/%s' % file.name # create an absolute/root url

    # poll listeners
    url_dict = {'url': file_url}
    sendfile_url_for_file.send(
        None, file=file,
        product=dl_product.downloadable_product,
        url_dict=url_dict,
    )
    # url may have changed; update it
    file_url = url_dict['url']

    # get file name from url
    file_name = os.path.basename(file_url)

    dl_product.num_attempts += 1
    dl_product.save()
    del request.session['download_key']
    response = HttpResponse()
    # For Nginx
    response['X-Accel-Redirect'] = file_url
    # For Apache and Lighttpd v1.5
    response['X-Sendfile'] = file_url
    # For Lighttpd v1.4
    response['X-LIGHTTPD-send-file'] = file_url
    response['Content-Disposition'] = "attachment; filename=%s" % file_name
    response['Content-length'] =  file.size
    contenttype, encoding = mimetypes.guess_type(file_name)
    if contenttype:
        response['Content-type'] = contenttype
    return response



from django import forms
from django.contrib.auth.models import User
from django.db.models import Q
from django.forms.extras.widgets import SelectDateWidget
from django.utils.translation import ugettext_lazy as _, ugettext
from l10n.models import Country
from livesettings import config_value, config_get_group, SettingNotSet
from satchmo_store.contact.models import Contact, AddressBook, PhoneNumber, Organization, ContactRole
from satchmo_store.shop.models import Config
from signals_ahoy.signals import form_init, form_initialdata, form_postsave
import datetime
import logging
import signals

log = logging.getLogger('satchmo_store.contact.forms')

selection = ''

class ProxyContactForm(forms.Form):
    def __init__(self, *args, **kwargs):
        self._contact = kwargs.pop('contact', None)
        super(ProxyContactForm, self).__init__(*args, **kwargs)

class ContactInfoForm(ProxyContactForm):
    email = forms.EmailField(max_length=75, label=_('Email'), required=False)
    title = forms.CharField(max_length=30, label=_('Title'), required=False)
    first_name = forms.CharField(max_length=30, label=_('First Name'), required=False)
    last_name = forms.CharField(max_length=30, label=_('Last Name'), required=False)
    phone = forms.CharField(max_length=30, label=_('Phone'), required=False)
    addressee = forms.CharField(max_length=61, label=_('Addressee'), required=False)
    organization = forms.CharField(max_length=50, label=_('Organization'), required=False)
    street1 = forms.CharField(max_length=30, label=_('Street'), required=False)
    street2 = forms.CharField(max_length=30, required=False)
    city = forms.CharField(max_length=30, label=_('City'), required=False)
    state = forms.CharField(max_length=30, label=_('State'), required=False)
    postal_code = forms.CharField(max_length=10, label=_('ZIP code/Postcode'), required=False)
    copy_address = forms.BooleanField(label=_('Shipping same as billing?'), required=False)
    ship_addressee = forms.CharField(max_length=61, label=_('Addressee'), required=False)
    ship_street1 = forms.CharField(max_length=30, label=_('Street'), required=False)
    ship_street2 = forms.CharField(max_length=30, required=False)
    ship_city = forms.CharField(max_length=30, label=_('City'), required=False)
    ship_state = forms.CharField(max_length=30, label=_('State'), required=False)
    ship_postal_code = forms.CharField(max_length=10, label=_('ZIP code/Postcode'), required=False)
    next = forms.CharField(max_length=40, widget=forms.HiddenInput(), required=False)

    def __init__(self, *args, **kwargs):
        initial = kwargs.get('initial', {})
        form_initialdata.send(self.__class__, form=self, initial=initial, contact = kwargs.get('contact', None))
        kwargs['initial'] = initial
        
        shop = kwargs.pop('shop', None)
        shippable = kwargs.pop('shippable', True)
        super(ContactInfoForm, self).__init__(*args, **kwargs)
        if not shop:
            shop = Config.objects.get_current()
        self._shop = shop
        self._shippable = shippable

        self.required_billing_data = config_value('SHOP', 'REQUIRED_BILLING_DATA')
        self._local_only = shop.in_country_only
        areas = shop.areas()
        self.enforce_state = config_value('SHOP','ENFORCE_STATE')

        if shop.in_country_only and areas and areas.count()>0:
            areas = [(area.abbrev or area.name, area.name) for area in areas]
            areas.insert(0,('',_("---Please Select---")))
            billing_state = (self._contact and getattr(self._contact.billing_address, 'state', None)) or selection
            shipping_state = (self._contact and getattr(self._contact.shipping_address, 'state', None)) or selection
            if self.enforce_state:
                self.fields['state'] = forms.ChoiceField(choices=areas, initial=billing_state, label=_('State'))
                self.fields['ship_state'] = forms.ChoiceField(choices=areas, initial=shipping_state, required=False, label=_('State'))
        
        self._default_country = shop.sales_country
        billing_country = (self._contact and getattr(self._contact.billing_address, 'country', None)) or self._default_country
        shipping_country = (self._contact and getattr(self._contact.shipping_address, 'country', None)) or self._default_country
        self.fields['country'] = forms.ModelChoiceField(shop.countries(), required=False, label=_('Country'), empty_label=None, initial=billing_country.pk)
        self.fields['ship_country'] = forms.ModelChoiceField(shop.countries(), required=False, label=_('Country'), empty_label=None, initial=shipping_country.pk)
        
        for fname in self.required_billing_data:
            if fname == 'country' and self._local_only:
                continue
            self.fields[fname].required = True
                
        # slap a star on the required fields
        for f in self.fields:
            fld = self.fields[f]
            if fld.required:
                fld.label = (fld.label or f) + '*'
        log.info('Sending form_init signal: %s', self.__class__)
        form_init.send(self.__class__, form=self)

    def _check_state(self, data, country):
        if country and self.enforce_state and country.adminarea_set.filter(active=True).count() > 0:
            if not data or data == selection:
                raise forms.ValidationError(
                    self._local_only and _('This field is required.') \
                               or _('State is required for your country.'))
            if (country.adminarea_set
                        .filter(active=True)
                        .filter(Q(name__iexact=data)|Q(abbrev__iexact=data))
                        .count() != 1):
                raise forms.ValidationError(_('Invalid state or province.'))
        return data

                
    def clean_email(self):
        """Prevent account hijacking by disallowing duplicate emails."""
        email = self.cleaned_data.get('email', None)
        if self._contact:
            if self._contact.email and self._contact.email == email:
                return email
            users_with_email = Contact.objects.filter(email=email)
            if len(users_with_email) == 0:
                return email
            if len(users_with_email) > 1 or users_with_email[0].id != self._contact.id:
                raise forms.ValidationError(
                    ugettext("That email address is already in use."))
        return email
    
    def clean_postal_code(self):
        postcode = self.cleaned_data.get('postal_code')
        if not postcode and 'postal_code' not in self.required_billing_data:
            return postcode
        country = None
        
        if self._local_only:
            shop_config = Config.objects.get_current()
            country = shop_config.sales_country
        else:
            country = self.fields['country'].clean(self.data.get('country'))

        if not country:
            # Either the store is misconfigured, or the country was
            # not supplied, so the country validation will fail and
            # we can defer the postcode validation until that's fixed.
            return postcode
        
        return self.validate_postcode_by_country(postcode, country)
    
    def clean_state(self):
        data = self.cleaned_data.get('state')
        if self._local_only:
            country = self._default_country
        else:
            country = self.fields['country'].clean(self.data.get('country'))
            if country == None:
                raise forms.ValidationError(_('This field is required.'))
        self._check_state(data, country)
        return data

    def clean_addressee(self):
        if not self.cleaned_data.get('addressee'):
            first_and_last = u' '.join((self.cleaned_data.get('first_name', ''),
                                       self.cleaned_data.get('last_name', '')))
            return first_and_last
        else:
            return self.cleaned_data['addressee']
    
    def clean_ship_addressee(self):
        if not self.cleaned_data.get('ship_addressee') and \
                not self.cleaned_data.get('copy_address'):
            first_and_last = u' '.join((self.cleaned_data.get('first_name', ''),
                                       self.cleaned_data.get('last_name', '')))
            return first_and_last
        else:
            return self.cleaned_data['ship_addressee']
    
    def clean_country(self):
        if self._local_only:
            return self._default_country
        else:
            if not self.cleaned_data.get('country'):
                log.error("No country! Got '%s'" % self.cleaned_data.get('country'))
                raise forms.ValidationError(_('This field is required.'))
        return self.cleaned_data['country']
        
    def clean_ship_country(self):
        copy_address = self.fields['copy_address'].clean(self.data.get('copy_address'))
        if copy_address:
            return self.cleaned_data.get('country')
        if self._local_only:
            return self._default_country
        if not self._shippable:
            return self.cleaned_data.get('country')
        shipcountry = self.cleaned_data.get('ship_country')
        if not shipcountry:
            raise forms.ValidationError(_('This field is required.'))
        if config_value('PAYMENT', 'COUNTRY_MATCH'):
            country = self.cleaned_data.get('country')
            if shipcountry != country:
                raise forms.ValidationError(_('Shipping and Billing countries must match'))
        return shipcountry

    def ship_charfield_clean(self, field_name):
        if self.cleaned_data.get('copy_address'):
            self.cleaned_data['ship_' + field_name] = self.fields[field_name].clean(self.data.get(field_name))
            return self.cleaned_data['ship_' + field_name]
        else:
            val = self.fields['ship_' + field_name].clean(self.data.get('ship_' + field_name))
            if (not val) and field_name in ('street1', 'city', 'state', 'postal_code'):
                raise forms.ValidationError(_('This field is required.'))
            return val

    def clean_ship_street1(self):
        return self.ship_charfield_clean('street1')

    def clean_ship_street2(self):
        if self.cleaned_data.get('copy_address'):
            if 'street2' in self.cleaned_data:
                self.cleaned_data['ship_street2'] = self.cleaned_data.get('street2')
        return self.cleaned_data.get('ship_street2')

    def clean_ship_city(self):
        return self.ship_charfield_clean('city')

    def clean_ship_postal_code(self):
        code = self.ship_charfield_clean('postal_code')
        if not self._shippable:
            return code

        country = None
        
        if self._local_only:
            shop_config = Config.objects.get_current()
            country = shop_config.sales_country
        else:
            country = self.ship_charfield_clean('country')
        
        if not country:
            # Either the store is misconfigured, or the country was
            # not supplied, so the country validation will fail and
            # we can defer the postcode validation until that's fixed.
            return code
        
        return self.validate_postcode_by_country(code, country)
        
    def clean_ship_state(self):
        data = self.cleaned_data.get('ship_state')
        
        if self.cleaned_data.get('copy_address'):
            if 'state' in self.cleaned_data:
                self.cleaned_data['ship_state'] = self.cleaned_data['state']
            return self.cleaned_data['ship_state']

        if self._local_only:
            country = self._default_country
        else:
            country = self.ship_charfield_clean('country')

        self._check_state(data, country)
        return data
    
    def save(self, **kwargs):
        if not kwargs.has_key('contact'):
            kwargs['contact'] = None
        return self.save_info(**kwargs)
    
    def save_info(self, contact=None, **kwargs):
        """Save the contact info into the database.
        Checks to see if contact exists. If not, creates a contact
        and copies in the address and phone number."""
        
        if not contact:
            customer = Contact()
            log.debug('creating new contact')
        else:
            customer = contact
            log.debug('Saving contact info for %s', contact)

        data = self.cleaned_data.copy()

        country = data['country']
        if not isinstance(country, Country):
            country = Country.objects.get(pk=country)
            data['country'] = country
        data['country_id'] = country.id

        shipcountry = data['ship_country']
        if not isinstance(shipcountry, Country):
            shipcountry = Country.objects.get(pk=shipcountry)
            data['ship_country'] = shipcountry
        
        data['ship_country_id'] = shipcountry.id

        organization_name = data.pop('organization', None)
        if organization_name:
            org = Organization.objects.by_name(organization_name, create=True)
            customer.organization = org
        else:
            # in case customer wants to remove organization name from their profile
            customer.organization = None

        for field in customer.__dict__.keys():
            try:
                setattr(customer, field, data[field])
            except KeyError:
                pass

        if not customer.role:
            customer.role = ContactRole.objects.get(pk='Customer')

        customer.save()
        
        # we need to make sure we don't blindly add new addresses
        # this isn't ideal, but until we have a way to manage addresses
        # this will force just the two addresses, shipping and billing
        # TODO: add address management like Amazon.
        
        bill_address = customer.billing_address
        if not bill_address:
            bill_address = AddressBook(contact=customer)
                
        changed_location = False
        address_keys = bill_address.__dict__.keys()
        for field in address_keys:
            if (not changed_location) and field in ('state', 'country_id', 'city'):
                if getattr(bill_address, field) != data[field]:
                    changed_location = True
            try:
                setattr(bill_address, field, data[field])
            except KeyError:
                pass

        bill_address.is_default_billing = True
        
        copy_address = data['copy_address']

        ship_address = customer.shipping_address
        
        if copy_address:
            # make sure we don't have any other default shipping address
            if ship_address and ship_address.id != bill_address.id:
                ship_address.delete()
            bill_address.is_default_shipping = True

        bill_address.save()
        
        if not copy_address:
            if not ship_address or ship_address.id == bill_address.id:
                ship_address = AddressBook()
            
            for field in address_keys:
                ship_field = 'ship_' + field
                if (not changed_location) and field in ('state', 'country_id', 'city'):
                    if getattr(ship_address, field) != data[ship_field]:
                        changed_location = True
                try:
                    setattr(ship_address, field, data[ship_field])
                except KeyError:
                    pass
            ship_address.is_default_shipping = True
            ship_address.is_default_billing = False
            ship_address.contact = customer
            ship_address.save()
            
        if not customer.primary_phone:
            phone = PhoneNumber()
            phone.primary = True
        else:
            phone = customer.primary_phone
        phone.phone = data['phone']
        phone.contact = customer
        phone.save()
        
        form_postsave.send(ContactInfoForm, object=customer, formdata=data, form=self)
        
        if changed_location:
            signals.satchmo_contact_location_changed.send(self, contact=customer)
        
        return customer.id
        
    def validate_postcode_by_country(self, postcode, country):
        responses = signals.validate_postcode.send(self, postcode=postcode, country=country)
        # allow responders to reformat the code, but if they don't return
        # anything, then just use the existing code
        for responder, response in responses:
            if response:
                return response
                
        return postcode

class DateTextInput(forms.TextInput):
    def render(self, name, value, attrs=None):
        if isinstance(value, datetime.date):
            value = value.strftime("%m.%d.%Y")
        return super(DateTextInput, self).render(name, value, attrs)

class ExtendedContactInfoForm(ContactInfoForm):
    """Contact form which includes birthday and newsletter."""
    years_to_display = range(datetime.datetime.now().year-100,datetime.datetime.now().year+1) 
    dob = forms.DateField(widget=SelectDateWidget(years=years_to_display), required=False)
    newsletter = forms.BooleanField(label=_('Newsletter'), widget=forms.CheckboxInput(), required=False)



from django import template
import re
try:
    from django.utils.safestring import mark_safe
except ImportError:
    mark_safe = lambda s:s

register = template.Library()

def rfc3339_date(date):
    return date.strftime('%Y-%m-%dT%H:%M:%SZ')

register.filter('atom_date', rfc3339_date)

def atom_tag_uri(url, date=None):
    tag = re.sub('^https?://', '', url)
            
    if date:
        tag = re.sub('/', ',%s:/' % date.strftime('%Y-%m-%d'), tag, 1)
        
    tag = re.sub('#', '/', tag)
    return 'tag:' + tag
    
register.filter('atom_tag_uri', atom_tag_uri)

def feed_safe_name(name):
    return name.replace(' ', '_').lower()

register.filter('feed_safe_name', feed_safe_name)

GOOGLE_TAGS = ('actor', 'age', 'age_range', 'agent', 'area', 'artist', 'aspect_ratio', 
    'author', 'bathrooms', 'battery_life', 'bedrooms', 'binding', 'brand', 'broker', 
    'calories', 'capacity', 'cholesterol', 'color', 'color_output', 'condition', 
    'cooking_time', 'course', 'course_date_range', 'course_number', 'course_times', 
    'cuisine', 'currency', 'department', 'description', 'director', 'display_type', 
    'edition', 'education', 'employer', 'ethnicity', 'event_date_range', 'event_type', 
    'expiration_date', 'expiration_date_time', 'feature', 'fiber', 'film_type', 'focus_type', 
    'format', 'from_location', 'functions', 'gender', 'genre', 'heel_height', 'height', 
    'hoa_dues', 'id', 'image_link', 'immigration_status', 'installation', 'interested_in', 
    'isbn', 'job_function', 'job_industry', 'job_type', 'language', 'length', 'link', 
    'listing_status', 'listing_type', 'load_type', 'location', 'lot_size', 'made_in', 
    'main_ingredient', 'make', 'marital_status', 'material', 'meal_type', 'megapixels', 
    'memory_card_slot', 'mileage', 'mls_listing_id', 'mls_name', 'model', 'model_number', 
    'mpn', 'name_of_item_reviewed', 'news_source', 'occasion', 'occupation', 'open_house_date_range', 
    'operating_system', 'optical_drive', 'pages', 'payment_accepted', 'payment_notes', 'performer', 
    'pickup', 'platform', 'preparation_time', 'price', 'price_type', 'processor_speed', 'product_type', 
    'property_taxes', 'property_type', 'protein', 'provider_class', 'provider_name', 
    'provider_telephone_number', 'publication_name', 'publication_volume', 'publish_date', 
    'publisher', 'quantity', 'rating', 'recommended_usage', 'resolution', 'review_type', 
    'reviewer_type', 'salary', 'salary_type', 'saturated_fat', 'school', 'school_district', 
    'screen_size', 'service_type', 'servings', 'sexual_orientation', 'shipping', 'shoe_width', 
    'size', 'sleeps', 'sodium', 'style', 'subject', 'tax_percent', 'tax_region', 'tech_spec_link', 
    'title', 'to_location', 'total_carbs', 'total_fat', 'travel_date_range', 'university', 'upc', 
    'url_of_item_reviewed', 'vehicle_type', 'venue_description', 'venue_name', 'venue_type', 
    'venue_website', 'vin', 'weight', 'width', 'wireless_interface', 'year', 'zoning', 'zoom'
)   
 
def make_googlebase_option(opt, custom):
    """Convert an option into a tag.  First look to see if it is a predefined tag, 
    if it is, good, use it.  Otherwise make a custom tag."""
    
    custom = custom.lower() in ('true','t','1')
    return make_googlebase_tag(opt.option_group.name, opt.name,custom)

register.filter('make_googlebase_option', make_googlebase_option)

def make_googlebase_attribute(att, custom):
    """Convert an attribute into a tag.  First look to see if it is a predefined tag, 
       if it is, good, use it.  Otherwise make a custom tag."""
     
    custom = custom.lower() in ('true','t','1')
    return make_googlebase_tag(att.name, att.value, custom)

register.filter('make_googlebase_attribute', make_googlebase_attribute)

def make_googlebase_tag(key, val, custom):
    """Convert a key/val pair into a tag.  First look to see if it is a predefined tag, 
    if it is, good, use it.  Otherwise make a custom tag."""
    key = feed_safe_name(key)
    
    if key in GOOGLE_TAGS:
        tag = "<g:%s>%s</g:%s>"
    elif key.endswith('s') and key[:-1] in GOOGLE_TAGS:
        key = key[:-1]
        tag = "<g:%s>%s</g:%s>"
    elif custom:
        tag = "<c:%s:string>%s</c:%s:string>"
    else:
        tag = None
    
    if tag:
        return mark_safe(tag % (key, val, key))
    else:
        return ""
    
def stripspaces(s):
    s = re.sub(r'^\s+', '', s)
    s = re.sub(r'\s+$', '', s)
    s = s.replace('\n\n','\n')
    return s
        
register.filter('stripspaces', stripspaces)


# Copyright (C) 2005 by Async Open Source
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import pango
import gtk
from kiwi.utils import PropertyObject, gproperty

from gazpacho.properties import prop_registry
from gazpacho.loader.custom import WidgetAdapter as LoaderAdapter, \
     adapter_registry
from gazpacho.widgets.base.base import WidgetAdaptor

custom_xpm = [
    "8 8 2 1",
    "  c #d6d6d6",
    "o c #bbbbbb",
    "        ",
    " oooooo ",
    " o    o ",
    " o    o ",
    " o    o ",
    " o    o ",
    " oooooo ",
    "        ",
    ]

MIN_WIDTH = MIN_HEIGHT = 20

# GObject
class Custom(PropertyObject, gtk.DrawingArea):
    gproperty('int1', int, nick='Integer 1', default=-1, minimum=-1)
    gproperty('int2', int, nick='Integer 2', default=-1, minimum=-1)
    gproperty('string1', str, nick='String 1')
    gproperty('string2', str, nick='String 2')
    gproperty('creation-function', str, nick='Creation function')
    gproperty('last-modification-time', str)

    def __init__(self, name=None, **kwargs):
        gtk.DrawingArea.__init__(self)
        PropertyObject.__init__(self, **kwargs)
        self.connect('notify::string1', lambda self, *a: self.queue_draw())

        if name:
            self.set_name(name)

        self._layout = self.create_pango_layout("")
        self._layout.set_font_description(
            pango.FontDescription("sans serif 12"))
        self._pixmap = None

        self.set_flags(self.flags() | gtk.CAN_FOCUS)

    def prop_set_visible(self, v):
        # Evil evil bug in PropertyObject, needs to
        # be investigated closer.
        self.set_property('visible', v)
        return v

    def do_realize(self):
        self.set_flags(self.flags() | gtk.REALIZED)

        events = (gtk.gdk.EXPOSURE_MASK |
                  gtk.gdk.BUTTON_PRESS_MASK |
                  gtk.gdk.POINTER_MOTION_MASK)

        colormap = self.get_colormap()
        self.window = gtk.gdk.Window(self.get_parent_window(),
                                     x=self.allocation.x,
                                     y=self.allocation.y,
                                     width=self.allocation.width,
                                     height=self.allocation.height,
                                     window_type=gtk.gdk.WINDOW_CHILD,
                                     wclass=gtk.gdk.INPUT_OUTPUT,
                                     visual=self.get_visual(),
                                     colormap=colormap,
                                     event_mask=self.get_events() | events)

        self.window.set_user_data(self)
        self.style.attach(self.window)
        self.style.set_background(self.window, gtk.STATE_NORMAL)

        if not self._pixmap:
            pixmap, _ = gtk.gdk.pixmap_colormap_create_from_xpm_d(
                None, colormap, None, custom_xpm)

            self._pixmap = pixmap

        self.window.set_back_pixmap(self._pixmap, False)

    def do_size_allocate(self, allocation):
        self.allocation = allocation
        if self.flags() & gtk.REALIZED:
            self.window.move_resize(*allocation)

    def do_size_request(self, requisition):
        requisition.width = MIN_WIDTH
        requisition.height = MIN_HEIGHT

    def do_expose_event(self, event):
        light_gc = self.style.light_gc[gtk.STATE_NORMAL]
        dark_gc = self.style.dark_gc[gtk.STATE_NORMAL]
        w, h = event.window.get_size()

        # These lines make the Placeholder look like a button
        event.window.draw_line(light_gc, 0, 0, w - 1, 0)
        event.window.draw_line(light_gc, 0, 0, 0, h - 1)
        event.window.draw_line(dark_gc, 0, h -1, w - 1, h - 1)
        event.window.draw_line(dark_gc, w - 1, 0, w - 1, h - 1)

        text = "Custom"
        if self.string1:
            text += ": %s" % self.string1
        self._layout.set_text(text)

        fontw, fonth = self._layout.get_pixel_size()
        self.style.paint_layout(self.window, self.state, False,
                                event.area, self, "label",
                                (w - fontw) / 2,
                                (h - fonth) / 2,
                                self._layout)
        return False

# Adapter
class CustomAdaptor(WidgetAdaptor):
    type = Custom
    name = 'Custom'

# Loader
class CustomLoaderAdapter(LoaderAdapter):
    object_type = Custom
    def construct(self, name, gtype, properties):
        return Custom()
adapter_registry.register_adapter(CustomLoaderAdapter)

CUSTOM_TYPE = 'gazpacho+widgets+base+custom+Custom'
prop_registry.override_simple(CUSTOM_TYPE + '::int1',
                              translatable=False, minimum=-1)
prop_registry.override_simple(CUSTOM_TYPE + '::int2',
                              translatable=False, minimum=-1)
prop_registry.override_simple(CUSTOM_TYPE + '::string1',
                              translatable=False)
prop_registry.override_simple(CUSTOM_TYPE + '::string2',
                              translatable=False)
prop_registry.override_simple(CUSTOM_TYPE + '::creation-function',
                              translatable=False)
prop_registry.override_simple(CUSTOM_TYPE + '::last-modification-time',
                              translatable=False, editable=False)



#!/usr/bin/env python

import re, logging
from threading import local
from importlib import import_module
from django.core.exceptions import ImproperlyConfigured, ViewDoesNotExist
from django.utils.datastructures import MultiValueDict
from django.utils.encoding import iri_to_uri, force_text, smart_str
from django.utils.regex_helper import normalize
from threading import currentThread
from functools import wraps

try:
    from django.utils.lru_cache import lru_cache
    def memoize(func, cache, num_args):
        return lru_cache()(func)
except ImportError:
    from django.utils.functional import memoize

log = logging.getLogger(__name__)

try:
    reversed
except NameError:
    from django.utils.itercompat import reversed     # Python 2.3 fallback
    from sets import Set as set

_resolver_cache = {} # Maps URLconf modules to RegexURLResolver instances.
_callable_cache = {} # Maps view and url pattern names to their view functions.

# SCRIPT_NAME prefixes for each thread are stored here. If there's no entry for
# the current thread (which is the only one we ever access), it is assumed to
# be empty.
_prefixes = {}
namespaces = {}

_urlconfs = local()

class NoReverseMatch(BaseException):
    # Don't make this raise an error when used in a template.
    silent_variable_failure = True

def get_namespace(viewname):
    if not ':' in viewname:
        ns = ''
        view = viewname
    else:    
        parts = viewname.split(':')
        parts.reverse()
        view = parts[0]
        path = parts[1:]
        ns = path[-1]
    return ns, view
get_namespace = memoize(get_namespace, namespaces, 1) 

def get_callable(lookup_view, can_fail=True):
    """
    Convert a string version of a function name to the callable object.

    If the lookup_view is not an import path, it is assumed to be a URL pattern
    label and the original string is returned.

    If can_fail is True, lookup_view might be a URL pattern label, so errors
    during the import fail and the string is returned.

    """
    if not callable(lookup_view):
        try:
            mod_name, func_name = get_mod_func(str(lookup_view))
            lookup_view = getattr(import_module(mod_name), func_name)
        except (ImportError, AttributeError):
            if can_fail: raise
        except UnicodeEncodeError:
            pass
    return lookup_view

get_callable = memoize(get_callable, _callable_cache, 1)

def get_resolver(urlconf):
    if urlconf is None:
        from django.conf import settings
        urlconf = settings.ROOT_URLCONF
    return RegexURLResolver(r'^/', urlconf)
get_resolver = memoize(get_resolver, _resolver_cache, 1)

def get_urlconf(default=None):
    return getattr(_urlconfs, "value", default)

def set_urlconf(urlconf_name):
    if urlconf_name:
        _urlconfs.value = urlconf_name
    else:
        if hasattr(_urlconfs, "value"):
            del _urlconfs.value

def get_mod_func(callback):
    # Converts 'django.views.news.stories.story_detail' to
    # ['django.views.news.stories', 'story_detail']
    try:
        dot = callback.rindex('.')
    except ValueError:
        return callback, ''
    return callback[:dot], callback[dot+1:]

class RegexURLPattern(object):
    def __init__(self, regex, callback, default_args=None, name=''):
        # regex is a string representing a regular expression.
        # callback is either a string like 'foo.views.news.stories.story_detail'
        # which represents the path to a module and a view function name, or a
        # callable object (view).
        self.regex = re.compile(regex, re.UNICODE)
        
        if callable(callback):
            self._callback = callback
        else:
            self._callback = None
            self._callback_str = callback

        self.default_args = default_args or {}
        self.name = name
    
    def __repr__(self):
        return '<%s %s %s>' % (self.__class__.__name__, self.name, self.regex.pattern)

    def add_prefix(self, prefix):
        """
        Adds the prefix string to a string-based callback.
        """
        if not prefix or not hasattr(self, '_callback_str'):
            return
        self._callback_str = prefix + '.' + self._callback_str

    def _get_namespace(self):

        return str(self.name)

    namespace = property(_get_namespace)

    def resolve(self, path):
        match = self.regex.search(force_text(path))
        if match:
            # If there are any named groups, use those as kwargs, ignoring
            # non-named groups. Otherwise, pass all non-named arguments as
            # positional arguments.
            kwargs = match.groupdict()
            if kwargs:
                args = ()
            else:
                args = match.groups()
            # In both cases, pass any extra_kwargs as **kwargs.
            kwargs.update(self.default_args)

            return (self.callback, args, kwargs)

    def _get_callback(self):
        
        if self._callback is not None:

            return self._callback
        try:
            mod_name, func_name = get_mod_func(str(self._callback_str))
            self._callback = getattr(import_module(mod_name), func_name)
        except (ImportError, AttributeError) as e:
            raise ViewDoesNotExist("Tried %s in module %s. Error was: %s" % (func_name, mod_name, str(e)))
        else:
            assert callable(self._callback) is True, 'not a callable: %s' % func_name
            return self._callback
    callback = property(_get_callback)

class RegexURLResolver(object):
    def __init__(self, regex, urlconf_name, default_kwargs=None, app_name=None, namespace=None, debug=False):
        # regex is a string representing a regular expression.
        # urlconf_name is a string representing the module containing URLconfs.
        self.regex = re.compile(regex, re.UNICODE)
        self.urlconf_name = urlconf_name
        if not isinstance(urlconf_name, str):
            self._urlconf_module = self.urlconf_name
        self.callback = None
        self.default_kwargs = default_kwargs or {}
        self.app_name = app_name
        self.namespace = namespace
        self._reverse_dict = {}
        self.debug = debug

    def __repr__(self):
        return '<%s %s %s>' % (self.__class__.__name__, self.urlconf_name, self.regex.pattern)

    def _get_reverse_dict(self):
        # a test case would be great 
        if hasattr(self, '_reverse_dict'):
            lookups = MultiValueDict(self._reverse_dict)
            for pattern in reversed([item for item in self.url_patterns]):
                if pattern is not None:
                    p_pattern = pattern.regex.pattern
                    if p_pattern.startswith('^'):
                        p_pattern = p_pattern[1:]
                else:
                    p_pattern = None
                if isinstance(pattern, RegexURLResolver):
                    parent = normalize(pattern.regex.pattern)
                    for name in pattern.reverse_dict:
                        for matches, pat in pattern.reverse_dict.getlist(name):
                            new_matches = []
                            for piece, p_args in parent:
                                new_matches.extend([(piece + suffix, p_args + args) for (suffix, args) in matches])
                            lookups.appendlist(name, (new_matches, p_pattern + pat))
                else:
                    if p_pattern:
                        bits = normalize(p_pattern)
                        #lookups.appendlist(pattern.callback, (bits, p_pattern))
                        if hasattr(pattern, 'name'):
                            lookups.appendlist(pattern.name, (bits, p_pattern))

            self._reverse_dict = lookups
        return self._reverse_dict
    reverse_dict = property(_get_reverse_dict)

    def resolve(self, path):
        match = self.regex.search(force_text(path))
        if match:
            #if self.debug: 
            #    log.debug("Got a match: %s"%path)
            new_path = path[match.end():]
            for pattern in self.url_patterns:
                try:
                    if pattern is not None:
                        
                        sub_match = pattern.resolve(new_path)
                except NoReverseMatch:
                    #log.debug(e)
                    #log.debug("caught a unknown path: %s"%new_path)
                    sub_match = None
                else:
                    if sub_match:
                        #if self.debug:
                        #    log.debug("Got sub_match: %r"%repr(sub_match))
                        sub_match_dict = dict([(k, v) for k, v in match.groupdict().items()])
                        sub_match_dict.update(self.default_kwargs)
                        for k, v in sub_match[2].items():
                            sub_match_dict[k] = v
                        return (sub_match[0], sub_match[1], sub_match_dict)
            raise NoReverseMatch('Resource not found: %s'%new_path)
        return None

    def _get_urlconf_module(self):
        try:
            return self._urlconf_module
        except AttributeError:
            self._urlconf_module = import_module(self.urlconf_name)
            return self._urlconf_module
    urlconf_module = property(_get_urlconf_module)

    def _get_url_patterns(self):
        patterns = getattr(self.urlconf_module, "urlpatterns", self.urlconf_module)
        try:
            iter(patterns)
        except TypeError:
            raise ImproperlyConfigured("The included urlconf %s doesn't have any "
                "patterns in it" % self.urlconf_name)
        return patterns
    url_patterns = property(_get_url_patterns)

    def _resolve_special(self, view_type):
        callback = getattr(self.urlconf_module, 'handler%s' % view_type, None)
        #if not callback: 
 		#    # Lazy import, since urls.defaults imports this file
        #    # See http://code.djangoproject.com/attachment/ticket/5350/urlresolvers.patch
 		#    from django.conf.urls import defaults 
 		#    callback=getattr(defaults, 'handler%s' % view_type) 
        
        mod_name, func_name = get_mod_func(callback)
        try:
            return getattr(import_module(mod_name), func_name), {}
        except (ImportError, AttributeError) as e:
            raise ViewDoesNotExist("Tried %s. Error was: %s" % (callback, str(e)))

    def resolve404(self):
        return self._resolve_special('404')

    def resolve500(self):
        return self._resolve_special('500')

    def reverse(self, lookup_view, *args, **kwargs):
        if args and kwargs:
            raise ValueError("Don't mix *args and **kwargs in call to reverse()!")
        #try:
        #    lookup_view = get_callable(lookup_view, True)
        #except (ImportError, AttributeError), e:
        #    raise NoReverseMatch("Error importing '%s': %s." % (lookup_view, e))       
        possibilities = self.reverse_dict.getlist(lookup_view)
        for possibility, pattern in possibilities:
            # result is callable
            for result, params in possibility:
                if args:
                    if len(args) != len(params):
                        continue
                    unicode_args = [force_text(val) for val in args]
                    candidate =  result % dict(zip(params, unicode_args))
                else:
                    if set(kwargs.keys()) != set(params):
                        continue
                    unicode_kwargs = dict([(k, force_text(v)) for (k, v) in kwargs.items()])
                    candidate = result % unicode_kwargs
                if re.search(u'^%s' % pattern, candidate, re.UNICODE):
                    return candidate
        raise NoReverseMatch("Reverse for '%s' with arguments '%s' and keyword "
                "arguments '%s' not found." % (lookup_view, args, kwargs))

def resolve(path, urlconf=None):
    return get_resolver(urlconf).resolve(path)

def reverse(viewname, urlconf=None, args=None, kwargs=None, prefix=None, current_app=None):
    args = args or []
    kwargs = kwargs or {}
    if prefix is None:
        prefix = get_script_prefix()
    # get the namespace here
    ns, view = get_namespace(viewname)
    #print "namespace: %s"% ns
    return iri_to_uri(u'%s%s' % (prefix, get_resolver(urlconf).reverse(viewname,
            *args, **kwargs)))

def clear_url_caches():
    global _resolver_cache
    global _callable_cache
    _resolver_cache.clear()
    _callable_cache.clear()

def set_script_prefix(prefix):
    """
    Sets the script prefix for the current thread.
    """
    if not prefix.endswith('/'):
        prefix += '/'
    _prefixes[currentThread()] = prefix

def get_script_prefix():
    """
    Returns the currently active script prefix. Useful for client code that
    wishes to construct their own URLs manually (although accessing the request
    instance is normally going to be a lot cleaner).
    """
    return _prefixes.get(currentThread(), u'/')

def is_valid_path(path, urlconf=None):
    """
    Returns True if the given path resolves against the default URL resolver,
    False otherwise.

    This is a convenience method to make working with "is this a match?" cases
    easier, avoiding unnecessarily indented try...except blocks.
    """
    try:
        resolve(path, urlconf)
        return True
    except NoReverseMatch:
        return False



# -*- coding: UTF-8 -*-
from django.http import Http404
import keyedcache
import random
from django.test import TestCase
import re
import time

CACHE_HIT=0

def cachetest(a,b,c):
    global CACHE_HIT
    CACHE_HIT += 1
    r = [random.randrange(0,1000) for x in range(0,3)]
    ret = [r, a + r[0], b + r[1], c + r[2]]
    return ret

cachetest = keyedcache.cache_function(2)(cachetest)
    
class DecoratorTest(TestCase):
    
    def testCachePut(self):
        d = cachetest(1,2,3)
        self.assertEqual(CACHE_HIT,1)
        
        d2 = cachetest(1,2,3)
        self.assertEqual(CACHE_HIT,1)
        self.assertEqual(d, d2)

        seeds = d[0]
        self.assertEqual(seeds[0] + 1, d[1])
        self.assertEqual(seeds[1] + 2, d[2])
        self.assertEqual(seeds[2] + 3, d[3])
        
        time.sleep(3)
        d3 = cachetest(1,2,3)
        self.assertEqual(CACHE_HIT,2)
        self.assertNotEqual(d, d3)
        
    def testDeleteCachedFunction(self):
        orig = cachetest(10,20,30)
        keyedcache.cache_delete_function(cachetest)
        after = cachetest(10,20,30)
        self.assertNotEqual(orig,keyedcache)

class CachingTest(TestCase):
    
    def testCacheGetFail(self):
        try:
            keyedcache.cache_get('x')
            self.fail('should have raised NotCachedError')
        except keyedcache.NotCachedError:
            pass
            
    def testCacheGetOK(self):
        one = [1,2,3,4]
        keyedcache.cache_set('ok', value=one, length=2)
        two = keyedcache.cache_get('ok')
        self.assertEqual(one, two)
        
        time.sleep(5)
        try:
            three = keyedcache.cache_get('ok')
            self.fail('should have raised NotCachedError, got %s' % three)
        except keyedcache.NotCachedError:
            pass
        
    def testCacheGetDefault(self):
        chk = keyedcache.cache_get('default',default='-')
        self.assertEqual(chk, '-')


    def testDelete(self):
        keyedcache.cache_set('del', value=True)
        
        for x in range(0,10):
            keyedcache.cache_set('del', 'x', x, value=True)
            for y in range(0,5):
                keyedcache.cache_set('del', 'x', x, 'y', y, value=True)

        # check to make sure all the values are in the cache
        self.assert_(keyedcache.cache_get('del', default=False))
        for x in range(0,10):
            self.assert_(keyedcache.cache_get('del', 'x', x, default=False))
            for y in range(0,5):
                self.assert_(keyedcache.cache_get('del', 'x', x, 'y', y, default=False))

        # try to delete just one
        killed = keyedcache.cache_delete('del','x',1)
        self.assertEqual([keyedcache.CACHE_PREFIX + "::del::x::1"], killed)
        self.assertFalse(keyedcache.cache_get('del', 'x', 1, default=False))
        
        # but the others are still there
        self.assert_(keyedcache.cache_get('del', 'x', 2, default=False))

        # now kill all of del::x::1
        killed = keyedcache.cache_delete('del','x', 1, children=True)
        for y in range(0,5):
            self.assertFalse(keyedcache.cache_get('del', 'x', 1, 'y', y, default=False))
        
        # but del::x::2 and children are there
        self.assert_(keyedcache.cache_get('del','x',2,'y',1, default=False))
        
        # kill the rest
        killed = keyedcache.cache_delete('del', children=True)
        self.assertFalse(keyedcache.cache_get('del',default=False))
        for x in range(0,10):
            self.assertFalse(keyedcache.cache_get('del', 'x', x, default=False))
            for y in range(0,5):
                self.assertFalse(keyedcache.cache_get('del', 'x', x, 'y', y, default=False))


class TestCacheDisable(TestCase):
    
    def testDisable(self):
        keyedcache.cache_set('disabled', value=False)
        v = keyedcache.cache_get('disabled')
        self.assertEqual(v, False)
        
        keyedcache.cache_enable(False)
        keyedcache.cache_set('disabled', value=True)
        try:
            keyedcache.cache_get('disabled')
            self.fail('should have raised NotCachedError')
        except keyedcache.NotCachedError, nce:
            key = keyedcache.cache_key('disabled')
            self.assertEqual(nce.key, key)
            
        keyedcache.cache_enable()
        v2 = keyedcache.cache_get('disabled')
        # should still be False, since the cache was disabled
        self.assertEqual(v2, False)

class TestKeyMaker(TestCase):
    
    def testSimpleKey(self):
        v = keyedcache.cache_key('test')
        self.assertEqual(v, keyedcache.CACHE_PREFIX + '::test')
        
    def testDualKey(self):
        v = keyedcache.cache_key('test', 2)
        self.assertEqual(v, keyedcache.CACHE_PREFIX + '::test::2')

    def testPairedKey(self):
        v = keyedcache.cache_key('test', more='yes')
        self.assertEqual(v, keyedcache.CACHE_PREFIX + '::test::more::yes')
        
    def testPairedDualKey(self):
        v = keyedcache.cache_key('test', 3, more='yes')
        self.assertEqual(v, keyedcache.CACHE_PREFIX + '::test::3::more::yes')


        



from payment.models import PaymentOption, CreditCardDetail
from django.contrib import admin


class CreditCardDetail_Inline(admin.StackedInline):
    model = CreditCardDetail
    extra = 1

class PaymentOptionOptions(admin.ModelAdmin):
    list_display = ['optionName','description','active']
    ordering = ['sortOrder']

admin.site.register(PaymentOption, PaymentOptionOptions)




#!/usr/bin/env python
# Copyright (c) Etienne Robillard 2007-2011 <erob@gthcfoundation.org>
# All rights reserved.
# <LICENSE=ISC>

import logging
logging.basicConfig(level=logging.DEBUG)

import logging.handlers 

# default logger for this app will be syslog
rootLogger = logging.getLogger(__name__)

#syslogHandler = logging.handlers.SysLogHandler(
#        address="/dev/log", facility=16
#        )
#rootLogger.addHandler(syslogHandler)
#rootLogger.debug('this is a test msg sent to the syslog daemon!')

def log_to_syslog(s, emitter=rootLogger.debug
    ):
    try:
        emitter(s)
    except:
        #problem emitting log record..
        raise
    return None

logger_func = lambda msg: log_to_syslog(msg)




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
from notmm.utils.configparse import setup_all
global_conf = dict()
setup_all(__name__, global_conf)
setup_all('httpserver', global_conf) # add httpserver support
#print global_conf[__name__]

from wikiapp.controllers import MoinMoinController

try:
    default_instance = global_conf[__name__].get('default_instance')
except KeyError:
    default_instance = None




from django import template

register = template.Library()

def addressblock(address):
    """Output an address as a HTML formatted text block"""
    return {"address" : address}

register.inclusion_tag('contact/_addressblock.html')(addressblock)

def contact_for_user(user):
    if not user.is_anonymous() and user.contact_set.count() > 0:
        return user.contact_set.all()[0]
    return None

register.filter('contact_for_user')(contact_for_user)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from notmm.utils.urlmap import RegexURLMap, url

urlpatterns = RegexURLMap()
urlpatterns.add_routes('',
        # Frontdoor (a simple generic view)
        (r'^index.html|$', 'sandbox.views.index', {
            'template_name': 'sandbox/test.mako',
            'charset': 'utf-8' # must specify mimetype or charset
        })
)
urlpatterns.include('sandbox.config.extras')



# from livesettings import *
# 
# SHIP_MODULES = config_get('SHIPPING', 'MODULES')
# SHIP_MODULES.add_choice(('shipping.modules.dummy', 'Dummy Shipping'))
# SHIPPING_GROUP = config_get_group('SHIPPING')
# 
# config_register_list(
# DecimalValue(SHIPPING_GROUP,
#     'FLAT_RATE',
#     description=_("Flat shipping"),
#     requires=SHIPPING_ACTIVE,
#     requiresvalue='shipping.modules.flat',
#     default="4.00"),
# )



import gobject
import gtk

from gazpacho.app.bars import bar_manager
from gazpacho.clipboard import clipboard
from gazpacho.constants import STOCK_SIZEGROUP
from gazpacho.i18n import _
from gazpacho.placeholder import Placeholder
from gazpacho.sizegroupeditor import add_sizegroup_gadgets
from gazpacho.gadget import Gadget
from gazpacho.signalhandlers import SignalHandlerStorage

class UIMState(object):
    """
    The UIMState controls the state of the UIManager. It's responsible
    for enabling and disabling action as well as add and remove ui
    definitions.

    The UIMState is responsible for the basic functionallity such as
    Save, Close, Undo and Redo actions. It should be extended and
    customized to controll other actions as well.

    It is possible to have more than one UIMState but only one should
    be enabled at one time.
    """

    def __init__(self):
        """
        Initialize the uim state.
        """
        self._enabled = False
        self._project = None
        # Project dependent handlers
        self.handlers = SignalHandlerStorage()
        self._merge_ids = []
    #
    # Public methods
    #

    def enable(self):
        """
        Enable the state object.
        """
        self._enabled = True
        self.update_uim()
        self.update_state()

    def disable(self):
        """
        Disable the state object.
        """
        self._enabled = False
        self.update_uim()
        self.update_state()

    def set_project(self, project):
        """
        Set the current project.

        @param project: the new project
        @type project: L{gazpacho.project.Project}
        """
        # Disconnect handlers
        self.handlers.disconnect_all()
        self._project = project
        self.update_state()

        if not self._project:
            return

        # Connect handlers
        self.handlers.connect(project, 'changed',
                              self._on_project_changed)
        self.handlers.connect(project.undo_stack, 'changed',
                              self._on_undo_stack_changed)

    def update_state(self):
        """
        Make sure that the state of all actions are up to date.
        """
        group = bar_manager.get_group('ContextActions')
        group.set_sensitive(self._project is not None)

        if not self._enabled:
            return

        if not self._project:
            self._set_undo_redo_labels()
            return

        self._update_save_action(self._project)
        self._update_undo_redo_actions(self._project.undo_stack)

    def merge_ui(self, ui_string):
        """
        Merge a ui definition. It's currently only possibe to merge
        one ui definition, if you want to merge another you have to
        remove the first.

        @param ui_string: the ui definition
        @type ui_string: str
        """
        merge_id = bar_manager.add_ui_from_string(ui_string)
        self._merge_ids.append(merge_id)

    def remove_ui(self):
        """
        Remove ui that has previously been merged. If nothing has been
        merge nothing will happen.
        """
        for merge_id in self._merge_ids:
            bar_manager.remove_ui(merge_id)
        self._merge_ids = []

    #
    # Private methods
    #

    def update_uim(self):
        """
        Update the UIManager. This will add or remove ui definitions
        as well as action groups. Note that this isn't really part of
        the public API but should be overridden by the subclasses.
        """
        raise NotImplementedError(
            "This method has to be overridden by subclasses")

    def _update_undo_redo_actions(self, undo_stack):
        """
        Update the undo/redo actions.

        @param undo_stack: the undo/redo stack
        @type undo_stack: L{gazpacho.project.UndoRedoStack}
        """
        undo_info = redo_info = None
        if self._project:
            undo_info = undo_stack.get_undo_info()
            redo_info = undo_stack.get_redo_info()

        bar_manager.set_action_prop('Undo', sensitive=undo_info is not None)
        bar_manager.set_action_prop('Redo', sensitive=redo_info is not None)
        self._set_undo_redo_labels(undo_info, redo_info)

    def _set_undo_redo_labels(self, undo_text=None, redo_text=None):
        """
        Set the undo and redo labels. If no text has been specified a
        default label will be used to indicate that there are no undo
        or redo available.

        @param undo_text: the text that describes the undo action
        @type undo_text: str
        @param redo_text: the text that describes the redo action
        @type redo_text: str
        """
        if undo_text is None:
            undo_text = _('Nothing')

        if redo_text is None:
            redo_text = _('Nothing')

        bar_manager.set_action_prop('Undo',
                                    label=_('_Undo: %s') % undo_text,
                                    short_label=_('_Undo'))

        bar_manager.set_action_prop('Redo',
                                    label=_('_Redo: %s') % redo_text,
                                    short_label=_('_Redo'))

    def _update_save_action(self, project):
        """
        Update the save action.

        @param project: the current project
        @type project: L{gazpacho.project.Project}
        """
        bar_manager.set_action_prop('Save', sensitive=self._project.changed)

    #
    # Signal handlers
    #

    def _on_project_changed(self, project):
        """
        Callback for the project's changed signal that is emitted when
        the state of the project has changed.

        @param project: the project that has changed
        @type project: L{gazpacho.project.Project}
        """
        if self._enabled:
            self._update_save_action(project)

    def _on_undo_stack_changed(self, undo_stack):
        """
        Callback for the undo/redo stack's changed signal.

        @param undo_stack: the undo/redo stack
        @type undo_stack: L{gazpacho.project.UndoRedoStack}
        """
        if self._enabled:
            self._update_undo_redo_actions(undo_stack)


ACTION_UI_STRING = """
<ui>
  <menubar name="MainMenu">
    <menu action="ObjectMenu">
      <menuitem action="AddAction"/>
      <menuitem action="AddActionGroup"/>
      <menuitem action="EditAction"/>
    </menu>
  </menubar>
  <toolbar name="MainToolbar">
    <toolitem action="AddAction"/>
  </toolbar>
</ui>
"""

ACTION_POPUP_UI_STRING = """
<ui>
  <popup name="ActionPopup">
    <menuitem action="AddAction"/>
    <menuitem action="AddActionGroup"/>
    <menuitem action="EditAction"/>
    <menuitem action="Delete"/>
  </popup>
</ui>
"""

class ActionUIMState(UIMState):
    """
    The ActionUIMState is responsible for actions that has to do with
    the GActionView, i.e. adding, removing and editing actions and
    action groups.
    """

    def __init__(self, view):
        """
        Initialize the action uim state.

        @param view: the action view
        @type view: L{gazpacho.actioneditor.GActionsView}
        """
        UIMState.__init__(self)

        self._view = view
        self._action_group = self._create_action_group()

        # Connect handlers (the normal way)
        view.connect('selection-changed', self._on_view_selection_changed)

    #
    # Public methods
    #

    def update_state(self):
        """
        Make sure that the state of all actions are up to date.
        """
        UIMState.update_state(self)
        self._action_group.set_sensitive(self._project is not None)

        if not (self._project and self._enabled):
            return

        self._update_action_actions()

    #
    # Private  methods
    #

    def update_uim(self):
        """
        Update the UIManager. This will add or remove ui definitions
        as well as action groups.
        """
        if self._enabled:
            if not bar_manager.has_group(self._action_group.get_name()):
                bar_manager.add_action_group(self._action_group)
            self.merge_ui(ACTION_UI_STRING)
            self.merge_ui(ACTION_POPUP_UI_STRING)
        else:
            self.remove_ui()
            group_name = self._action_group.get_name()
            if bar_manager.has_group(group_name):
                bar_manager.remove_action_group(group_name)

    def _create_action_group(self):
        """
        Create an action group for the action and action group
        specific actions.

        @return: the action group
        @rtype: gtk.ActionGroup
        """
        group = gtk.ActionGroup('ActionActions')
        group.add_actions((('Delete', gtk.STOCK_DELETE, None, '<control>D',
                            _('Delete'), self._delete_cb),))

        action = gtk.Action('AddAction',
                            _('_Add Action...'),
                            _('Add an action'),
                            gtk.STOCK_ADD)
        action.connect('activate', self._add_action_cb)
        action.set_property('short-label', _('Action'))
        group.add_action(action)

        action = gtk.Action('AddActionGroup',
                            _('Add Action _Group...'),
                            _('Add an action group'),
                            gtk.STOCK_ADD)
        action.connect('activate', self._add_action_group_cb)
        action.set_property('short-label', _('Action Group'))
        group.add_action(action)

        action = gtk.Action('EditAction',
                            _('_Edit...'),
                            _('Edit selected action or action group'),
                            gtk.STOCK_EDIT)
        action.connect('activate', self._edit_action_cb)
        action.set_property('short-label', _('Edit'))
        group.add_action(action)

        return group

    def _update_action_actions(self):
        """
        Update the action and action group actions.
        """
        if not self._project:
            return

        bar_manager.set_action_prop('AddActionGroup', sensitive=True)

        if (self._view.get_selected_action() or
            self._view.get_selected_action_group()):
            sensitive = True
        else:
            sensitive = False

        bar_manager.set_action_props(('AddAction', 'Delete', 'EditAction'),
                                     sensitive=sensitive)

    #
    # Signal handlers
    #

    def _on_view_selection_changed(self, actionview, item):
        """
        Callback that is called when the selection in the action view
        has changed.

        @param actionview: the action view
        @type actionview: L{gazpacho.actioneditor.GActionsView}
        @param item: the selected gaction or gaction group
        @type item: L{gazpacho.gaction.GActionGroup} or
          L{gazpacho.gaction.GAction}
        """
        if self._enabled:
            self._update_action_actions()

    #
    # Action callbacks
    #

    def _delete_cb(self, action):
        gaction = self._view.get_selected_action()
        if gaction is not None:
            self._view.remove_action(gaction)

    def _add_action_cb(self, action):
        gaction = self._view.get_selected_action()
        self._view.add_action(gaction)

    def _add_action_group_cb(self, action):
        self._view.add_action_group()

    def _edit_action_cb(self, action):
        gaction = self._view.get_selected_action()
        if gaction is not None:
            self._view.edit_action(gaction)

    def _delete_action_cb(self, action):
        gaction = self._view.get_selected_action()
        if gaction is not None:
            self._view.remove_action(gaction)


SIZEGROUP_POPUP_UI_STRING = """
<ui>
  <popup name="SizeGroupPopup">
    <menuitem action="Delete"/>
  </popup>
</ui>
"""

class SizeGroupUIMState(UIMState):
    """
    The SizeGroupUIMState is responsible for actions that has to do
    with the SizeGroupView, i.e. manipulating sizegroups and their
    widgets.
    """

    def __init__(self, view):
        """
        Initialize the sizegroup uim state.

        @param view: the sizegroup view
        @type view: L{gazpacho.sizegroupeditor.SizeGroupView}
        """
        UIMState.__init__(self)

        self._view = view
        self._action_group = self._create_sizegroup_action_group()

        # Connect handlers (the normal way)
        view.connect('selection-changed', self._on_view_selection_changed)

    #
    # Public methods
    #

    def update_state(self):
        """
        Make sure that the state of all actions are up to date.
        """
        UIMState.update_state(self)
        self._action_group.set_sensitive(self._project is not None)

        if not (self._project and self._enabled):
            return

        enabled = self._view.has_selected()
        self._update_sizegroup_actions(enabled)

    #
    # Private  methods
    #

    def update_uim(self):
        """
        Update the UIManager. This will add or remove ui definitions
        as well as action groups.
        """
        if self._enabled:
            if not bar_manager.has_group(self._action_group.get_name()):
                bar_manager.add_action_group(self._action_group)
            self.merge_ui(SIZEGROUP_POPUP_UI_STRING)
        else:
            self.remove_ui()
            group_name = self._action_group.get_name()
            if bar_manager.has_group(group_name):
                bar_manager.remove_action_group(group_name)

    def _create_sizegroup_action_group(self):
        """
        Create an action group for the sizegroup specific actions.

        @return: the action group
        @rtype: gtk.ActionGroup
        """
        group = gtk.ActionGroup('Edit sizegroup actions')
        group.add_actions((('Delete', gtk.STOCK_DELETE, None, '<control>D',
                            _('Delete'), self._delete_cb),))
        return group

    def _update_sizegroup_actions(self, enabled):
        """
        Update the actions related to sizegroup manipulation.

        @param enabled: whether the actions should be enabled or not
        @type enabled: bool
        """
        bar_manager.set_action_prop('Delete', sensitive=enabled)

    #
    # Signal handlers
    #

    def _on_view_selection_changed(self, view, item):
        """
        Callback that is called when the selected item in the
        sizegroup view has changed.

        @param view: the sizegroup view
        @type view: L{gazpacho.sizegroupeditor.SizeGroupView}
        @param item: the selected widget or sizegroup
        @type item: L{gazpacho.sizegroup.GSizeGroup} or
          L{gazpacho.gadget.Gadget}
        """
        self._update_sizegroup_actions(bool(item))

    #
    # Action callbacks
    #

    def _delete_cb(self, action):
        self._view.remove_selected_item()


WIDGET_UI_STRING = """
<ui>
  <menubar name="MainMenu">
    <menu action="ObjectMenu">
      <menuitem action="AddSizeGroupWidgets"/>
    </menu>
  </menubar>
  <toolbar name="MainToolbar">
    <toolitem action="AddSizeGroupWidgets"/>
  </toolbar>
</ui>
"""

class WidgetUIMState(UIMState):
    """
    The WidgetUIMState is responsible for actions that has to do with
    widgets, i.e. adding, removing, cut and paste and so on.
    """

    def __init__(self):
        """
        Initialize the widget uim state.
        """
        UIMState.__init__(self)

        self._action_group = self._create_widget_action_group()

        # Connect handlers (the normal way)
        clipboard.connect('selection_changed',
                          self._on_clipboard_selection_changed)

    #
    # Public methods
    #

    def set_project(self, project):
        """
        Set the current project.

        @param project: the new project
        @type project: L{gazpacho.project.Project}
        """
        UIMState.set_project(self, project)

        if not project:
            return

        self.handlers.connect(project.selection, 'selection_changed',
                              self._on_project_selection_changed)

    def update_state(self):
        """
        Make sure that the state of all actions are up to date.
        """
        UIMState.update_state(self)
        self._action_group.set_sensitive(self._project is not None)

        if not (self._project and self._enabled):
            return

        selection = self._project.selection
        self._update_edit_actions(selection)
        self._update_sizegroup_actions(selection)

    #
    # Private  methods
    #

    def update_uim(self):
        """
        Update the UIManager. This will add or remove ui definitions
        as well as action groups.
        """
        if self._enabled:
            if not bar_manager.has_group(self._action_group.get_name()):
                bar_manager.add_action_group(self._action_group)
            self.merge_ui(WIDGET_UI_STRING)
        else:
            self.remove_ui()
            group_name = self._action_group.get_name()
            if bar_manager.has_group(group_name):
                bar_manager.remove_action_group(group_name)

    def _create_widget_action_group(self):
        """
        Create an action group for the widget specific actions.

        @return: the action group
        @rtype: gtk.ActionGroup
        """
        group = gtk.ActionGroup('WidgetActions')

        # We override the default cut, copy, paste and delete
        group.add_actions(
            (('Cut', gtk.STOCK_CUT, None, None, _('Cut'), self._cut_cb),
             ('Copy', gtk.STOCK_COPY, None, None, _('Copy'), self._copy_cb),
             ('Paste', gtk.STOCK_PASTE, None, None, _('Paste'),
              self._paste_cb),
             ('Delete', gtk.STOCK_DELETE, None, '<control>D', _('Delete'),
              self._delete_cb)
             )
            )

        action = gtk.Action('AddSizeGroupWidgets',
                            _('Add Size Group _Widgets'),
                            _('Add the selected widgets to a size group'),
                            STOCK_SIZEGROUP)
        action.connect('activate', self._add_sizegroup_gadgets_cb)
        action.set_property('short-label', _('Group'))
        group.add_action(action)

        return group

    def _update_paste_action(self, selection, clipboard_item):
        """
        Update the paste action.

        @param selection: the selected widgets
        @type selection: list
        @param clipboard_item: the selected item on the clipboard
        @type clipboard_item: L{gazpacho.clipboard.ClipboardItem}
        """
        sensitive = False
        if clipboard_item:
            # We can always paste a toplevel
            if clipboard_item.is_toplevel:
                sensitive = True

            # columns can be pasted in tree views
            elif (len(selection) == 1
                  and isinstance(selection[0], gtk.TreeView)
                  and gobject.type_is_a(clipboard_item.type,
                                        gtk.TreeViewColumn)):
                sensitive = True

            # otherwise we need a placeholder
            elif (len(selection) == 1
                  and isinstance(selection[0], Placeholder)):
                sensitive = True

        bar_manager.set_action_prop('Paste', sensitive=sensitive)


    def _update_edit_actions(self, selection):
        """
        Update the actions in the edit group.

        @param selection: the selected widgets
        @type selection: list
        """
        if len(selection) == 1:
            widget = selection[0]
            gadget = Gadget.from_widget(widget)

            # Placeholders cannot be cut or copied but sometimes deleted
            if isinstance(widget, Placeholder):
                bar_manager.set_action_props(('Copy', 'Cut'),
                                             sensitive=False)
                bar_manager.set_action_prop('Delete',
                                            sensitive=widget.is_deletable())

            # Internal children cannot be cut, copied or deleted.
            elif gadget.internal_name:
                bar_manager.set_action_props(('Copy', 'Cut', 'Delete'),
                                             sensitive=False)

            else:
                bar_manager.set_action_props(('Copy', 'Cut', 'Delete'),
                                             sensitive=True)
        else:
            bar_manager.set_action_props(('Copy', 'Cut', 'Delete'),
                                         sensitive=False)

        # Unless the widget is toplevel it can only be pasted on a placeholder
        item = clipboard.get_selected_item()
        self._update_paste_action(selection, item)


    def _update_sizegroup_actions(self, selection):
        """
        Update the sizegroup actions.

        @param selection: the selected widgets
        @type selection: list
        """
        sensitive = True

        if not selection:
            sensitive = False

        for widget in selection:
            if isinstance(widget, (gtk.Window, Placeholder)):
                sensitive = False

        bar_manager.set_action_prop('AddSizeGroupWidgets', sensitive=sensitive)

    #
    # Signal handlers
    #

    def _on_clipboard_selection_changed(self, clipboard, item):
        """
        Callback for the clipboard's selection-changed signal.

        @param clipboard: the clipboard
        @type clipboard: L{gazpacho.clipboard.Clipboard}
        @param item: the newly selected item
        @type item: L{gazpacho.clipboard.ClipboardItem}
        """
        if not self._enabled:
            return

        selection = self._project.selection
        self._update_paste_action(selection, item)

    def _on_project_selection_changed(self, project):
        """
        Callback for when the selected widget in this project has changed.

        @param project: the current project
        @type project: L{gazpacho.project.Project}
        """
        if not self._enabled:
            return

        selection = self._project.selection
        self._update_edit_actions(selection)
        self._update_sizegroup_actions(selection)

    #
    # Action callbacks
    #

    def _cut_cb(self, action):
        gadget = Gadget.from_widget(self._project.selection[0])
        clipboard.cut(gadget)

    def _copy_cb(self, action):
        gadget = Gadget.from_widget(self._project.selection[0])
        clipboard.copy(gadget)

    def _paste_cb(self, action):
        placeholder = None
        if len(self._project.selection) == 1:
            placeholder = self._project.selection[0]
        clipboard.paste(placeholder, self._project)

    def _delete_cb(self, action):
        self._project.delete_selection()

    def _add_sizegroup_gadgets_cb(self, action):
        """
        Callback that will add the selected widgets to a sizegroup
        specified by the user.

        @param action: the action that triggered the callback
        @type action: L{gtk.Action}
        """
        add_sizegroup_gadgets(self._project)



from django import forms
from django import http
from django.core import urlresolvers
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
from satchmo_store.mail import send_store_mail
from satchmo_store.shop.signals import contact_sender
import logging

log = logging.getLogger('satchmo_store.shop.views')

# Choices displayed to the user to categorize the type of contact request.
email_choices = (
    ("General Question", _("General question")),
    ("Order Problem", _("Order problem")),
)

class ContactForm(forms.Form):
    name = forms.CharField(label=_("Name"), max_length=100)
    sender = forms.EmailField(label=_("Email address"), max_length=75)
    subject = forms.CharField(label=_("Subject"))
    inquiry = forms.ChoiceField(label=_("Inquiry"), choices=email_choices)
    contents = forms.CharField(label=_("Contents"), widget=
        forms.widgets.Textarea(attrs={'cols': 40, 'rows': 5}))

def form(request):
    if request.method == "POST":
        form = ContactForm(request.POST)
        if form.is_valid():
            new_data = form.cleaned_data
            c = {
                'request_type': new_data['inquiry'],
                'name': new_data['name'],
                'email': new_data['sender'],
                'request_text': new_data['contents'] }
            subject = new_data['subject']
            send_store_mail(subject, c, 'shop/email/contact_us.txt',
                            send_to_store=True, sender=contact_sender)
            url = urlresolvers.reverse('satchmo_contact_thanks')
            return http.HttpResponseRedirect(url)
    else: #Not a post so create an empty form
        initialData = {}
        if request.user.is_authenticated():
            initialData['sender'] = request.user.email
            initialData['name'] = request.user.first_name + " " + request.user.last_name
        form = ContactForm(initial=initialData)

    return render_to_response('shop/contact_form.html', {'form': form},
                              context_instance=RequestContext(request))



"""
Associates products to each other for upselling purposes.
"""

# TODO:
# - Rename Upsell object to what it actually is, a CrossSell
# - Implement an Upsell which would *replace* the item.  Like a "supersize" concept.

from datetime import date
from decimal import Decimal, getcontext
from django.conf import settings
from django.db import models
from django.utils.translation import get_language, gettext_lazy as _
from django.utils.translation import ugettext, ugettext_lazy as _
from keyedcache.models import CachedObjectMixin
from product.models import Product
import datetime
import keyedcache
import logging

log = logging.getLogger('upsell.models')

UPSELL_CHOICES=(
    ('CHECKBOX_1_FALSE', _('Checkbox to add 1')),
    ('CHECKBOX_1_TRUE', _('Checkbox to add 1, checked by default')),
    ('CHECKBOX_MATCH_FALSE', _('Checkbox to match quantity')),
    ('CHECKBOX_MATCH_TRUE', _('Checkbox to match quantity, checked by default')),
    ('FORM', _('Form with 0 quantity')),
)

class Upsell(models.Model, CachedObjectMixin):
    
    target = models.ManyToManyField(Product, verbose_name=_('Target Product'), 
        related_name="upselltargets",
        help_text = _("The products for which you want to show this goal product as an Upsell."))
    
    goal = models.ForeignKey(Product, verbose_name=_('Goal Product'), 
        related_name="upsellgoals")
    
    create_date = models.DateField(_("Creation Date"))

    style = models.CharField(_("Upsell Style"), choices=UPSELL_CHOICES,
        default='CHECKBOX_1_FALSE', max_length=20)
        
    notes = models.TextField(_('Notes'), blank=True, null=True,
        help_text = _("Internal notes"))
        
                
    def _description(self):
        """Get the description, looking up by language code, falling back intelligently.
        """
        language_code = get_language()

        try:
            trans = self.cache_get(trans=language_code)

        except keyedcache.NotCachedError, e:
            trans = self._find_translation(language_code)

        if trans:
            return trans.description
        else:
            return ""
            
    description = property(fget=_description)

    def _find_translation(self, language_code):
        c = self.translations.filter(languagecode__exact = language_code)
        ct = c.count()

        if not c or ct == 0:
            pos = language_code.find('-')
            if pos>-1:
                short_code = language_code[:pos]
                log.debug("%s: Trying to find root language content for: [%s]", self, short_code)
                c = self.translations.filter(languagecode__exact = short_code)
                ct = c.count()
                if ct>0:
                    log.debug("%s: Found root language content for: [%s]", self, short_code)

        if not c or ct == 0:
            #log.debug("Trying to find default language content for: %s", self)
            c = self.translations.filter(languagecode__istartswith = settings.LANGUAGE_CODE)
            ct = c.count()

        if not c or ct == 0:
            #log.debug("Trying to find *any* language content for: %s", self)
            c = self.translations.all()
            ct = c.count()

        if ct > 0:
            trans = c[0]
        else:
            trans = None

        self.cache_set(trans=language_code, value=trans)

        return trans
        
    def is_form(self):
        """Returns true if the style is a FORM"""
        return self.style.startswith("FORM")
        
    def is_qty_one(self):
        """Returns true if this style has a '1' in the center field"""
        parts = self.style.split("_")
        return parts[1] == '1'

    def is_checked(self):
        """Returns true if this style ends with TRUE"""
        return self.style.endswith('TRUE')
        
    def __unicode__(self):
        return u"Upsell for %s" % self.goal
        
    def save(self, **kwargs):
        self.create_date = datetime.date.today()
        self.cache_delete()
        super(Upsell, self).save(**kwargs)
        self.cache_set()
        return self
        
    class Meta:
        ordering = ('goal',)
        
class UpsellTranslation(models.Model):

    menu = models.ForeignKey(Upsell, related_name="translations")
    languagecode = models.CharField(_('language'), max_length=10, 
        choices=settings.LANGUAGES, default=settings.LANGUAGES[0][0])
    description = models.TextField(_('Description'), blank=True)

    class Meta:
        ordering=('languagecode', )



""" image related filters """

##################################################
## DEPENDENCIES ##

from django import template
from django.conf import settings
from django.template import TemplateSyntaxError
from satchmo_utils.thumbnail.utils import make_thumbnail, get_image_size
register = template.Library()
##################################################
## FILTERS ##

def thumbnail(url, args=''):
    """ Returns thumbnail URL and create it if not already exists.

.. note:: requires PIL_,
    if PIL_ is not found or thumbnail can not be created returns original URL.

.. _PIL: http://www.pythonware.com/products/pil/

Usage::

    {{ url|thumbnail:"width=10,height=20" }}
    {{ url|thumbnail:"width=10" }}
    {{ url|thumbnail:"height=20" }}

Parameters:

width
    requested image width

height
    requested image height

Image is **proportionally** resized to dimension which is no greather than ``width x height``.

Thumbnail file is saved in the same location as the original image
and his name is constructed like this::

    %(dirname)s/%(basename)s_t[_w%(width)d][_h%(height)d].%(extension)s

or if only a width is requested (to be compatibile with admin interface)::

    %(dirname)s/%(basename)s_t%(width)d.%(extension)s

"""
    
    kwargs = {}
    if args:
        if ',' not in args:
            # ensure at least one ','
            args += ','
        for arg in args.split(','):
            arg = arg.strip()
            if arg == '': continue
            kw, val = arg.split('=', 1)
            kw = kw.lower().encode('ascii')
            try:
                val = int(val) # convert all ints
            except ValueError:
                raise template.TemplateSyntaxError, "thumbnail filter: argument %r is invalid integer (%r)" % (kw, val)
            kwargs[kw] = val
    
    if ('width' not in kwargs) and ('height' not in kwargs):
        raise template.TemplateSyntaxError, "thumbnail filter requires arguments (width and/or height)"
    
    ret = make_thumbnail(url, **kwargs)
    if ret is None:
        ret = url

    if not ret.startswith(settings.MEDIA_URL):
        ret = settings.MEDIA_URL + ret

    return ret
   
#
register.filter('thumbnail', thumbnail)

def image_width(url):
    """ Returns image width.

Usage:
    {{ url|image_width }}
"""
    
    width, height = get_image_size(url)
    return width
#

register.filter('image_width', image_width)

def image_height(url):
    """ Returns image height.

Usage:
    {{ url|image_width }}
"""
    
    width, height = get_image_size(url)
    return height
#

register.filter('image_height', image_height)



"""New-style configuration variables should go in this file.

Once again, we're taking another route, but not reinventing any
wheel. 

To retain the original Django configuration scheme, use the 
following import statement ::

    >>> from configuration import settings 

In short, YMMV...

"""
from notmm.utils.django_settings import SettingsProxy

settings = SettingsProxy(autoload=True).get_settings()
settings.PROJECT_NAME = 'MightyHelloWorldApp'

__all__ = ('settings', )




# Copyright (C) 2004,2005 by SICEm S.L. and Imendio AB
#               2005,2006 by Async Open Source
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import gtk
import gobject
from kiwi.python import Settable
from kiwi.utils import gsignal, type_register
from kiwi.ui.dialogs import BaseDialog, open
from kiwi.ui.widgets.list import List, Column
from kiwi.ui.widgets.entry import ProxyEntry

from gazpacho.commandmanager import command_manager
from gazpacho.i18n import _
from gazpacho.properties import PropertyCustomEditor, UNICHAR_PROPERTIES, \
     prop_registry, UnsupportedProperty, CommandSetProperty, \
     CommandSetTranslatableProperty, PropertySetError
from gazpacho.signaleditor import SignalEditor
from gazpacho.stockicons import StockIconDialog

class IncompleteFunctions(Exception):
    pass

TABLE_ROW_SPACING = 2
TABLE_COLUMN_SPACING = 6

(EDITOR_INTEGER,
 EDITOR_FLOAT,
 EDITOR_DOUBLE) = list(range(3))

(FLAGS_COLUMN_SETTING,
 FLAGS_COLUMN_NAME,
 FLAGS_COLUMN_MASK) = list(range(3))

class EditorProperty:
    """ For every PropertyClass we have a EditorProperty that is basically
    an input (widget) for that PropertyClass. """

    wide_editor = False

    def __init__(self, property_class, app):
        # The class this property corresponds to
        self.property_class = property_class

        self._app = app
        self._project = app.get_current_project()

        # The widget that modifies this property, the widget can be a
        # GtkSpinButton, a GtkEntry, GtkMenu, etc. depending on the property
        # type.
        self.input = None

        # The loaded property
        self.property = None

        # If we're in the process of loading the property
        self.property_loading = False

        # We set this flag when we are loading a new Property into this
        # EditorProperty. This flag is used so that when we receive a "changed"
        # signal we know that nothing has really changed, we just loaded a new
        # gadget
        self.loading = False

        # the label to the left of the input
        label = gtk.Label("%s:" % property_class.label)
        label.set_alignment(1.0, 0.5)

        # we need to wrap the label in an event box to add tooltips
        eventbox = gtk.EventBox()
        eventbox.add(label)

        self.description = eventbox

    def __cmp__(self, other):
        return cmp(self.property_class.priority, other.property_class.priority)

    def load(self, widget):
        # Should be overridden
        return

    def external_set(self, value):
        """
        This is called when something external changes the property,
        Override this in a subclass and update the displayed content
        """

    # XXX: untested
    def set(self, value):
        try:
            cmd = CommandSetProperty(self.property, value)
            command_manager.execute(cmd, self._project)
        except PropertySetError:
	    # The value set by the user was not valid, so we
	    # have to restore it to the old value.
            self.external_set(self.property.value)

    def set_translatable_property(self, text, comment_text, translatable,
                                  has_context):
        cmd = CommandSetTranslatableProperty(
            self.property, text, comment_text, translatable, has_context)
        command_manager.execute(cmd, self._project)

    def _property_changed(self, prop):
        self.external_set(prop.get())

    def _start_load(self, widget):
        if self.property_class.child:
            prop = widget.get_child_prop(self.property_class.name)
        else:
            prop = widget.get_prop(self.property_class.name)
        prop.add_change_notify(self._property_changed)
        self.property = prop

        assert self.property
        self.property_loading = True

    def _finish_load(self):
        self.property_loading = False

    def _internal_load(self, value):
        """This is the virtual method that actually put the data in the editor
        widget.
        Most subclasses implements this (But not EditorPropertySpecial)
        """

class EditorPropertyNumeric(EditorProperty):

    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)
        self._value_changed_id = -1
        self._entry_changed_id = -1

    def load(self, widget):
        self._start_load(widget)
        self._internal_load(self.property.value)

        self.input.set_data('user_data', self)
        self._finish_load()

    def _internal_load(self, value):
        if self.property_class.name in UNICHAR_PROPERTIES:
            if isinstance(self.input, gtk.Editable):
                self.input.delete_text(0, -1)
                self.input.insert_text(value)
        #elif self.property_class.optional:
        #    button, spin = self.input.get_children()
        #    button.set_active(self.property.enabled)
        #    spin.set_sensitive(self.property.enabled)
        #    button.set_data('user_data', self)
        #    v = float(value)
        #    spin.set_value(v)
        elif value is not None:
            try:
                v = float(value)
            except ValueError:
                v = 0
            self.input.set_value(v)

    def _changed_unichar(self, entry):
        if self.property_loading:
            return

        # We need to handle the text as a unicode string. Note when
        # importing gtk the encoding will be forced to utf-8
        text = entry.get_chars(0, -1).decode()
        if text:
            self.set(text[0])

    def _create_input_unichar(self):
        entry = gtk.Entry()
        # it's to prevent spirious beeps ...
        entry.set_max_length(2)
        self._entry_changed_id = entry.connect('changed', self._changed_unichar)
        self.input = entry

    def _changed_numeric(self, spin):
        if self.property_loading:
            return

        numeric_type = spin.get_data('NumericType')
        if numeric_type == EDITOR_INTEGER:
            value = spin.get_value_as_int()
        elif numeric_type == EDITOR_FLOAT:
            value = spin.get_value()
        elif numeric_type == EDITOR_DOUBLE:
            value = spin.get_value()
        else:
            value = 0

        self.set(value)

    def _changed_enabled(self, check_button):
        if self.property_loading:
            return

        l = self.input.get_children()
        spin = l[1]
        state = check_button.get_active()
        spin.set_sensitive(state)
        prop = check_button.get_data('user_data')
        prop.property.enabled = state

    def _create_input_numeric(self, numeric_type):
        spin = gtk.SpinButton(self.property_class.get_adjustment(),
                              self.property_class.climb_rate,
                              self.property_class.digits)
        spin.set_data("NumericType", numeric_type)
        self._value_changed_id = spin.connect('value_changed', self._changed_numeric)

        # Some numeric types are optional, for example the default window size,
        # so they have a toggle button right next to the spin button
        #if self.property_class.optional:
        #    check = gtk.CheckButton()
        #    hbox = gtk.HBox()
        #    hbox.pack_start(check, False, False)
        #    hbox.pack_start(spin)
        #    check.connect('toggled', self._changed_enabled)
        #    self.input = hbox
        #else:
        self.input = spin

    def external_set(self, value):
        widget = self.input
        if isinstance(widget, gtk.SpinButton):
            widget.handler_block(self._value_changed_id)
            widget.set_value(value)
            widget.handler_unblock(self._value_changed_id)
        elif isinstance(self.input, gtk.Entry):
            widget.handler_block(self._entry_changed_id)
            widget.set_text(value)
            widget.handler_unblock(self._entry_changed_id)

class EditorPropertyInteger(EditorPropertyNumeric):
    def __init__(self, property_class, app):
        EditorPropertyNumeric.__init__(self, property_class, app)

        if self.property_class.name in UNICHAR_PROPERTIES:
            self._create_input_unichar()
        else:
            self._create_input_numeric(EDITOR_INTEGER)

class EditorPropertyFloat(EditorPropertyNumeric):
    def __init__(self, property_class, app):
        EditorPropertyNumeric.__init__(self, property_class, app)
        self._create_input_numeric(EDITOR_FLOAT)

class EditorPropertyDouble(EditorPropertyNumeric):
    def __init__(self, property_class, app):
        EditorPropertyNumeric.__init__(self, property_class, app)
        self._create_input_numeric(EDITOR_DOUBLE)

class EditorPropertyText(EditorProperty):
    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)

        self.input = gtk.HBox()
        # If the current data contains more than one line,
        # use a GtkTextView to display the data, otherwise just use a
        # normal GtkEntry
        if property_class.multiline:
            view = gtk.TextView()
            view.set_editable(True)
            text_buffer = view.get_buffer()
            text_buffer.connect('changed', self._on_text_buffer__changed)
            self.input.pack_start(view)
        else:
            self._entry = gtk.Entry()
            self._entry.connect('changed', self._on_entry__changed)
            self.input.pack_start(self._entry)

        if self.property_class.translatable:
            button = gtk.Button('...')
            button.connect('clicked', self._on_i18n_button__clicked)
            self.input.pack_start(button, False, False)

    def external_set(self, value):
        self.property_loading = True
        self._internal_load(value)
        self.property_loading = False

    def load(self, widget):
        self._start_load(widget)

        self._internal_load(self.property.value)

        self.input.set_data('user_data', self)

        self._finish_load()

    def _internal_load(self, value):
        # Text inputs are an entry or text view inside an hbox
        text_widget = self.input.get_children()[0]

        if isinstance(text_widget, gtk.Editable):
            pos = text_widget.get_position()
            text_widget.delete_text(0, -1)
            if value:
                text_widget.insert_text(value)

            text_widget.set_position(pos)

        elif isinstance(text_widget, gtk.TextView):
            text_buffer = text_widget.get_buffer()
            text_buffer.set_text(value)
        else:
            print ('Invalid Text Widget type')

    # Callbacks

    def _on_i18n_button__clicked(self, button):
        self._show_i18n_dialog()

    def _on_entry__changed(self, entry):
        if self.property_loading:
            return

        text = entry.get_chars(0, -1)
        self.set(text)

    def _on_text_buffer__changed(self, text_buffer):
        if self.property_loading:
            return

        start = text_buffer.get_iter_at_offset(0)
        end = text_buffer.get_iter_at_offset(text_buffer.get_char_count())
        text = text_buffer.get_text(start, end, False)
        self.set(text)

    # Private

    def _show_i18n_dialog(self):
        editor = self.input.get_toplevel()

        translatable = self.property.translatable
        has_context = self.property.has_i18n_context
        comment_text = self.property.i18n_comment

        dialog = BaseDialog(editor, _('Edit Text Property'),
                            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                                     gtk.STOCK_OK, gtk.RESPONSE_OK))

        dialog.set_has_separator(False)

        vbox = gtk.VBox(False, 6)
        vbox.set_border_width(8)
        vbox.show()

        dialog.vbox.pack_start(vbox, True, True, 0)

        # The text content
        label = gtk.Label()
        label.set_markup_with_mnemonic(_('_Text:'))
        label.show()
        label.set_alignment(0, 0.5)
        vbox.pack_start(label, False, False, 0)

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.set_size_request(400, 200)
        scrolled_window.show()

        vbox.pack_start(scrolled_window)

        text_view = gtk.TextView()
        text_view.show()
        label.set_mnemonic_widget(text_view)

        scrolled_window.add(text_view)

        text_buffer = text_view.get_buffer()

        text = self.property.value
        if text is not None:
            text_buffer.set_text(text)

        # Translatable and context prefix
        hbox = gtk.HBox(False, 12)
        hbox.show()

        vbox.pack_start(hbox, False, False, 0)

        translatable_button = gtk.CheckButton(_("T_ranslatable"))
        translatable_button.set_active(translatable)
        translatable_button.show()
        hbox.pack_start(translatable_button, False, False, 0)

        context_button = gtk.CheckButton(_("_Has context prefix"))
        context_button.set_active(has_context)
        context_button.show()
        hbox.pack_start(context_button, False, False, 0)

        # Comments
        label = gtk.Label()
        label.set_markup_with_mnemonic(_("Co_mments for translators:"))
        label.show()
        label.set_alignment(0, 0.5)
        vbox.pack_start(label, False, False, 0)

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.set_shadow_type(gtk.SHADOW_IN)
        scrolled_window.set_size_request(400, 200)
        scrolled_window.show()

        vbox.pack_start(scrolled_window)

        comment_view = gtk.TextView()
        comment_view.show()
        label.set_mnemonic_widget(comment_view)

        scrolled_window.add(comment_view)

        comment_buffer = comment_view.get_buffer()

        if comment_text is not None:
            comment_buffer.set_text(comment_text)

        res = dialog.run()
        if res == gtk.RESPONSE_OK:
            translatable = translatable_button.get_active()
            has_context = context_button.get_active()

            start, end = text_buffer.get_bounds()
            text = text_buffer.get_text(start, end, False)

            start, end = comment_buffer.get_bounds()
            comment_text = comment_buffer.get_text(start, end, False)

            self.set_translatable_property(text, comment_text, translatable,
                                           has_context)

            # Update the entry in the property editor
            #self.load(self.property.widget)

        dialog.destroy()

class EditorPropertyTextBrowse(EditorPropertyText):
    """
    EditorPropertyTextBrowse is an editor which behaves like a text entry, but
    it has an extra button on the right side which you can click so it opens up
    a file chooser dialog.
    To customize the types that can be loaded, override the get_filter() method
    """
    def __init__(self, property_class, app):
        EditorPropertyText.__init__(self, property_class, app)

        if property_class.translatable:
            raise TypeError(
                "You can't use a EditorPropertyTextBrowse on a translatable property")

        button = gtk.Button('...')
        button.connect('clicked', self._on_browse_button__clicked)
        self.input.pack_start(button, False, False)

    def _on_browse_button__clicked(self, button):
        self._show_browse_dialog()

    def _show_browse_dialog(self):
        filename = open("Select an image", parent=self.input.get_toplevel(),
                        filter=self.get_filter())
        if filename:
            self._entry.set_text(filename)

    def get_filter(self):
        """
        @returns: a gtk.FileFilter or None
        """

class EditorPropertyImage(EditorPropertyTextBrowse):
    """
    A property editor for loading images
    """
    def get_filter(self):
        filter = gtk.FileFilter()
        filter.set_name("Images")
        filter.add_mime_type("image/png")
        filter.add_mime_type("image/jpeg")
        filter.add_mime_type("image/gif")
        filter.add_mime_type("image/x-xpixmap")
        filter.add_pattern("*.png")
        filter.add_pattern("*.jpg")
        filter.add_pattern("*.gif")
        filter.add_pattern("*.xpm")
        return filter

class EditorPropertyStock(EditorPropertyText):
    """
    A property editor for stock icons
    """
    def __init__(self, property_class, app):
        EditorPropertyText.__init__(self, property_class, app)

        button = gtk.Button('...')
        button.connect('clicked', self._on_browse_button__clicked)
        self.input.pack_start(button, False, False)

    def _on_browse_button__clicked(self, button):
        self._show_stock_icon_dialog()

    def _show_stock_icon_dialog(self):
        dialog = StockIconDialog(parent=self.input.get_toplevel())
        dialog.select(self.property.value)
        response = dialog.run()
        if response == gtk.RESPONSE_OK:
            stock_id = dialog.get_selected()
            self._entry.set_text(stock_id)
        dialog.hide()

class EditorPropertyEnum(EditorProperty):
    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)

        model = gtk.ListStore(str, object)
        for name, value in self.property_class.get_choices():
            model.append((name, value))

        combo_box = gtk.ComboBox(model)
        renderer = gtk.CellRendererText()
        combo_box.pack_start(renderer)
        combo_box.set_attributes(renderer, text=0)
        combo_box.connect('changed', self._changed_enum)

        self.input = combo_box

    def load(self, widget):
        self._start_load(widget)
        self._internal_load(self.property.value)
        self._finish_load()

    def _internal_load(self, value):
        idx = 0
        for choice in self.property.get_choices():
            if choice[1] == value:
                break
            idx += 1
        self.input.set_active(idx)

    def _changed_enum(self, combo_box):
        if self.property_loading:
            return

        active_iter = combo_box.get_active_iter()
        model = combo_box.get_model()
        value = model[active_iter][1]
        self.set(value)

class EditorPropertyFlags(EditorProperty):
    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)

        self.input = List([Column('value', data_type=bool, editable=True),
                           Column('name', data_type=str),
                           Column('mask', data_type=str, visible=False)])
        self.input.get_treeview().set_headers_visible(False)
        self.input.connect('cell-edited', self._on_input__cell_edited)

        # Populate the model with the flags
        self.input.add_list(
            [Settable(value=False, name=name, mask=mask)
             for name, mask in property_class.get_choices()])
        self.input.show()

    def load(self, widget):
        self._start_load(widget)
        self._internal_load(self.property.value)
        self._finish_load()

    def _internal_load(self, value):
        for flag in self.input:
            flag.value = bool((value & flag.mask) == flag.mask)
            self.input.update(flag)

    # Called when the value column is edited, since it's the only
    # editable column.
    def _on_input__cell_edited(self, flags, flag, attribute):
        value = 0
        for flag in self.input:
            if flag.value:
                value |= flag.mask

        if value != self.property.value:
            self.set(value)

class EditorPropertyBoolean(EditorProperty):
    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)

        button = gtk.ToggleButton(_('No'))
        self._toggled_id = button.connect('toggled', self._changed_boolean)

        self.input = button

    def external_set(self, value):
        self.input.handler_block(self._toggled_id)
        self.input.set_active(value)
        self.input.handler_unblock(self._toggled_id)

    def load(self, widget):
        self._start_load(widget)
        self._internal_load(self.property.value)

        self.input.set_data('user_data', self)
        self._finish_load()

    def _internal_load(self, state):
        label = self.input.get_child()

        if state:
            value = 'yes'
        else:
            value = 'no'
        label.set_text(value)

        self.input.set_active(state)

    def _changed_boolean(self, button):
        if self.property_loading:
            return

        if button.get_active():
            value = 'yes'
        else:
            value = 'no'

        label = button.get_child()
        label.set_text(value)
        self.set(button.get_active())

class EditorPropertyCustomEditor(EditorProperty):
    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)

        self.custom_editor = self.property_class.custom_editor()
        self.input = self.custom_editor.get_editor_widget()

    def load(self, gadget):
        self._start_load(gadget)

        self.widget = gadget

        proxy = EditorValueProxy()
        proxy.connect('value-set', self._value_changed, gadget)
        proxy.connect('property-set', self._property_set, gadget)

        context = gadget.project.context
        self.custom_editor.update(context, gadget.widget, proxy)
        self._finish_load()

    def _value_changed(self, proxy, value, gadget):
        if self.property_loading:
            return

        self.set(value)

    def _property_set(self, proxy, prop_id, value, gadget):
        if self.property_class.child:
            property = gadget.get_child_prop(prop_id)
        else:
            property = gadget.get_prop(prop_id)

        if not property or self.property_loading:
            return

        self.set(value)

class EditorValueProxy(gobject.GObject):
    gsignal('value-set', object)
    gsignal('property-set', str, object)

    def set_value(self, value):
        self.emit('value-set', value)

    def set_property(self, prop, value):
        self.emit('property-set', prop, value)

if gtk.pygtk_version < (2, 8):
    gobject.type_register(EditorValueProxy)

# BUGS:
#   the cellrenderers show up twice
#   no way to remove the widget, eg set it to empty
#
class EditorPropertyObject(EditorProperty):
    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)

        self.property_class = property_class
        self.property_type = property_class.type
        self.project = app.get_current_project()

        self.model = gtk.ListStore(str, object)

        combo_box = gtk.ComboBox(self.model)
        renderer = gtk.CellRendererText()
        combo_box.pack_start(renderer)
        combo_box.set_attributes(renderer, text=0)
        combo_box.connect('changed', self._changed_enum)

        self.input = combo_box

    def load(self, widget):
        self._start_load(widget)
        current = self.property.get()

        self.model.clear()
        for pwidget in self.project.get_widgets():
            if isinstance(pwidget, gtk.Window):
                continue

            if not gobject.type_is_a(pwidget, self.property_type):
                continue

            iter = self.model.append((pwidget.name, pwidget))
            if pwidget == current:
                self.input.set_active_iter(iter)

        self._finish_load()

    def _changed_enum(self, combo_box):
        if self.property_loading:
            return

        active_iter = combo_box.get_active_iter()
        model = combo_box.get_model()
        widget = model[active_iter][1]
        self.set(widget)

class PropertyEditorDialog(BaseDialog):
    """This is a base abstract class for those editors that need to be
    opened in a dialog.
    """

    def __init__(self):
        """The widgets of the dialog are created in the virtual method
        _create_widgets.
        """
        BaseDialog.__init__(self,
                            buttons=(gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
        self.widget = None
        self.gadget = None
        self.proxy = None

        # avoid destrcution when clicking the x button
        self.connect('delete-event', self._on_delete_event)
        self.connect('response', self._on_response)

        self.set_resizable(False)

        self._create_widgets()

        self.vbox.show_all()

    def _create_widgets(self):
        """Subclasses should override this."""

    def set_gadget(self, gadget, proxy):
        """Every time the user clicks on a custom property editor button
        we are called with a different widget and proxy.
        """
        self.proxy = proxy
        self.gadget = gadget

    def _on_delete_event(self, dialog, event):
        self.hide()
        return True

    def _on_response(self, dialog, response_id):
        self.hide()

class PropertyCustomEditorWithDialog(PropertyCustomEditor):
    """A custom editor which is just a button that opens up a dialog"""

    # subclasses should probably want to override the dialog_class
    dialog_class = PropertyEditorDialog
    button_text = _('Edit...')

    def __init__(self):
        self.dialog = self.dialog_class()
        self.input = gtk.Button(self.button_text)
        self.input.connect('clicked', self.on_button_clicked)

        self.application_window = None
        self.proxy = None
        self.edited_gadget = None

    def get_editor_widget(self):
        return self.input

    def update(self, context, widget, proxy):
        from gazpacho.gadget import Gadget

        self.application_window = context.get_application_window()
        self.edited_gadget = Gadget.from_widget(widget)
        self.proxy = proxy

    def on_button_clicked(self, button):
        self.dialog.set_transient_for(self.application_window)
        self.dialog.set_gadget(self.edited_gadget, self.proxy)
        self.dialog.present()

class EditorPropertyName(EditorProperty):

    duplicate_name_msg = _("There is already a widget with this name")
    no_name_msg = _("The widget must have a name")

    def __init__(self, property_class, app):
        EditorProperty.__init__(self, property_class, app)

        self.input = ProxyEntry()
        self.input.set_property('data-type', str)
        self.input.connect('changed', self._changed_text)

    def load(self, widget):
        self._start_load(widget)
        self._internal_load(widget)
        self._finish_load()

    def _internal_load(self, widget):
        value = self.property.value
        pos = self.input.get_position()
        self.input.delete_text(0, -1)
        if value:
            self.input.insert_text(value)

        self.input.set_position(pos)

        # validate that the name exists and is unique
        if not value:
            self.input.set_invalid(self.no_name_msg)
        elif self._project.get_gadget_by_name(value) != widget:
            self.input.set_invalid(self.duplicate_name_msg)
        else:
            self.input.set_valid()

    def _changed_text(self, entry):
        """
        Update the name property of the widget if the name is valid.
        """
        if self.property_loading:
            return

        name = entry.get_chars(0, -1)
        if not name:
            self.input.set_invalid(self.no_name_msg)
        elif self._project.get_gadget_by_name(name):
            self.input.set_invalid(self.duplicate_name_msg)
        else:
            self.input.set_valid()
            self.set(name)


# this maps property types with EditorProperty classes
property_map = {
    gobject.TYPE_INT: EditorPropertyInteger,
    gobject.TYPE_UINT: EditorPropertyInteger,
    gobject.TYPE_FLOAT: EditorPropertyFloat,
    gobject.TYPE_DOUBLE: EditorPropertyDouble,
    gobject.TYPE_ENUM: EditorPropertyEnum,
    gobject.TYPE_FLAGS: EditorPropertyFlags,
    gobject.TYPE_BOOLEAN: EditorPropertyBoolean,
    gobject.TYPE_STRING: EditorPropertyText
    }

class EditorPage(gtk.Table):
    """
    For each widget type that we have modified, we create an EditorPage.
    An EditorPage is a gtk.Table subclass with all the widgets to edit
    a particular widget type
    """
    def __init__(self):
        gtk.Table.__init__(self, 1, 1, False)

        self._tooltips = gtk.Tooltips()
        self._rows = 0

        self.set_border_width(6)
        self.set_row_spacings(TABLE_ROW_SPACING)
        self.set_col_spacings(TABLE_COLUMN_SPACING)

    def _attach_row(self, label, editable):
        row = self._rows
        if label:
            self.attach(label, 0, 1, row, row+1,
                        xoptions=gtk.FILL, yoptions=gtk.FILL)
            self.attach(editable, 1, 2, row, row+1,
                        xoptions=gtk.EXPAND|gtk.FILL,
                        yoptions=gtk.EXPAND|gtk.FILL)
        else:
            self.attach(editable, 0, 2, row, row+1)
        self._rows += 1

    def append_item(self, editor):
        if editor.input:
            self._tooltips.set_tip(editor.description,
                                   editor.property_class.description)
            # TODO: Hmm.
            if editor.property_class.custom_editor.wide_editor:
                # no label for this editor since it need both columns
                # of the table
                description = None
            else:
                description = editor.description

        self._attach_row(description, editor.input)

    def append_name_field(self, widget_adaptor):
        # Class
        label = gtk.Label(_('Class')+':')
        label.set_alignment(1.0, 0.5)
        class_label = gtk.Label(widget_adaptor.editor_name)
        class_label.set_alignment(0.0, 0.5)
        self._attach_row(label, class_label)

    def reset(self):
        list(map(self.remove, self.get_children()))
        self._rows = 0

class PropertyEditor(gtk.Notebook):
    gsignal('add-signal', str, int, int, str)

    def __init__(self, app):
        gtk.Notebook.__init__(self)
        self.set_size_request(350, 300)

        self._app = app

        # The editor has (at this moment) four tabs this are pointers to the
        # vboxes inside each tab.
        self._vbox_gadget = self._notebook_page(_('Widget'))
        self._vbox_packing = self._notebook_page(_('Packing'))
        self._vbox_common = self._notebook_page(_('Common'))
        self._vbox_signals = self._notebook_page(_('Signals'))

        # Create the pages, one for widget properties and
        # one for the common ones
        self._widget_table = EditorPage()
        self._vbox_gadget.pack_start(self._widget_table, False)

        self._common_table = EditorPage()
        self._vbox_common.pack_start(self._common_table, False)

        self._packing_table = EditorPage()
        self._vbox_packing.pack_start(self._packing_table, False)

        # A handy reference to the widget that is loaded in the editor. None
        # if no widgets are selected
        self._loaded_gadget = None

        # A list of properties for the current widget
        # XXX: We might want to cache this
        self._widget_properties = []
        self._widget_editors = {}

        self._signal_editor = None

    def _notebook_page(self, name):
        vbox = gtk.VBox()

        scrolled_window = gtk.ScrolledWindow()
        scrolled_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled_window.add_with_viewport(vbox)
        scrolled_window.set_shadow_type(gtk.SHADOW_NONE)

        self.insert_page(scrolled_window, gtk.Label(name), -1)

        return vbox

    def do_add_signal(self, id_widget, type_widget, id_signal, callback_name):
        pass

    def _create_editor_property(self, property_class):
        if property_class.custom_editor != PropertyCustomEditor:
            return EditorPropertyCustomEditor(property_class, self._app)

        if property_class.editor:
            return property_class.editor(property_class, self._app)

        # Special case GObject subclasses
        if gobject.type_is_a(property_class.type, gobject.GObject):
            return EditorPropertyObject(property_class, self._app)

        if property_class.base_type in property_map:
            return property_map[property_class.base_type](property_class,
                                                          self._app)

    def _create_property_pages(self, gadget, adaptor):
        widget_table = self._widget_table
        common_table = self._common_table
        packing_table = self._packing_table
        parent = gadget.widget.get_parent()
        widget_table.reset()
        common_table.reset()
        packing_table.reset()

        widget_table.append_name_field(adaptor)

        # Put the property widgets on the right page
        widget_properties = []


        editors = self._widget_editors.get((adaptor.type_name, parent))
        if editors is None:
            # Append name to the widget table first
            widget_editors = self._widget_editors.setdefault(
                (adaptor.type_name, parent), [])

            if isinstance(gadget.widget, gtk.Widget):
                property_class = prop_registry.get_prop_type('GtkWidget',
                                                             'name')
                editor = EditorPropertyName(property_class, self._app)
                widget_editors.append((widget_table, editor))

            for property_class in prop_registry.list(adaptor.type_name,
                                                     parent):
                if not property_class.editable:
                    continue

                # Skip name, it's already added above
                name = property_class.name
                if name == 'name':
                    continue

                # Check the property too, sort of a hack
                if (name in gadget.properties and
                    not gadget.properties[name].editable):
                    continue

                if property_class.child:
                    page = packing_table
                elif property_class.owner_type in (gtk.Object.__gtype__,
                                                   gtk.Widget.__gtype__):
                    page = common_table
                else:
                    page = widget_table

                try:
                    editor = self._create_editor_property(property_class)
                    if not editor:
                        continue
                except UnsupportedProperty:
                    continue

                widget_editors.append((page, editor))

            editors = self._widget_editors.get((adaptor.type_name, parent))

        editors.sort()

        for page, editor in editors:
            page.append_item(editor)
            widget_properties.append(editor)

        self._widget_properties = widget_properties

        # XXX: Remove this, show all widgets individually instead
        widget_table.show_all()
        common_table.show_all()
        packing_table.show_all()

    def _create_signal_page(self):
        if self._signal_editor:
            return self._signal_editor

        self._signal_editor = SignalEditor(self, self._app)
        self._vbox_signals.pack_start(self._signal_editor)

    def _get_parent_types(self, widget):
        retval = [type(widget)]
        while True:
            parent = widget.get_parent()
            if not parent:
                return retval
            retval.append(type(parent))
            widget = parent

    def _needs_rebuild(self, gadget):
        """
        Return True if we need to rebuild the current property pages, False
        if it's not require.
        """

        if not self._loaded_gadget:
            return True

        # Check if we need to rebuild the interface, otherwise we might end
        # up with a (child) properties which does not belong to us
        # FIXME: This implementation is not optimal, in some cases it'll
        # rebuild even when it doesn't need to, a better way would be
        # to compare child properties.
        if (self._get_parent_types(self._loaded_gadget.widget) !=
            self._get_parent_types(gadget.widget)):
            return True

        return False

    def _load_gadget(self, gadget):
        if self._needs_rebuild(gadget):
            self._create_property_pages(gadget, gadget.adaptor)
            self._create_signal_page()

        for widget_property in self._widget_properties:
            widget_property.load(gadget)
        self._signal_editor.load_gadget(gadget)

        self._loaded_gadget = gadget

    def hide_content(self):
        self._set_visible(False)

    def _set_visible(self, value):
        for vbox in (self._vbox_gadget,
                     self._vbox_common,
                     self._vbox_packing):
            if value:
                vbox.show()
            else:
                vbox.hide()

    def display(self, gadget):
        "Display a widget in the editor"

        self._set_visible(True)

        # Skip widget if it's already loaded or None
        if self._loaded_gadget == gadget or not gadget:
            return

        self._load_gadget(gadget)

    def refresh(self, prop_name=None):
        """Reread properties and update the editor

        If prop_name is given only refresh that property editor
        """
        if not self._loaded_gadget:
            return

        if prop_name:
            for widget_property in self._widget_properties:
                if widget_property.property_class.name == prop_name:
                    widget_property.load(self._loaded_gadget)
        else:
            self._load_gadget(self._loaded_gadget)

    def get_loaded_gadget(self):
        return self._loaded_gadget

type_register(PropertyEditor)

